;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------

.module man_animations


.include "animations.h.s"
.include "../sprites/sprites.h.s"
.include "entity.h.s"
.include "cpctelera.h.s"

_animations_array_player::
    .dw man_anim_player_standing 
    .dw man_anim_player_death
    .dw man_anim_player_shooting
    .dw man_anim_player_down
    .dw man_anim_player_up
_animations_array_enemy_fire::
    .dw man_anim_enemy_fire_standing
    .dw man_anim_enemy_death
    .dw man_anim_enemy_death_damage
    .dw man_anim_enemy_fire_walking
_animations_array_enemy_water::
    .dw man_anim_enemy_water_standing
    .dw man_anim_enemy_death
    .dw man_anim_enemy_death_damage
    .dw man_anim_enemy_water_walking
_animations_array_enemy_electric::
    .dw man_anim_enemy_electric_standing
    .dw man_anim_enemy_death
    .dw man_anim_enemy_death_damage
    .dw man_anim_enemy_electric_walking
_animations_array_enemy_mixed::
    .dw man_anim_enemy_mixed_standing
    .dw man_anim_enemy_death
    .dw man_anim_enemy_death_damage
    .dw man_anim_enemy_mixed_walking
_animations_array_enemy_letter::
    .dw man_anim_enemy_letter_standing
    .dw man_anim_enemy_death
    .dw man_anim_enemy_death_damage
    .dw man_anim_enemy_letter_walking
_animations_array_enemy_heart::
    .dw man_anim_enemy_heart_standing
    .dw man_anim_enemy_death
    .dw man_anim_enemy_death_damage
    .dw man_anim_enemy_heart_walking

_animations_array_spell_fire::
    .dw man_anim_spell_fire
    .dw man_anim_spell_fire_destroy
_animations_array_spell_water::
    .dw man_anim_spell_water
    .dw man_anim_spell_water_destroy
_animations_array_spell_electric::
    .dw man_anim_spell_electric
    .dw man_anim_spell_electric_destroy
_animations_array_spell_letter::
    .dw man_anim_spell_letter
    .dw man_anim_spell_letter_destroy
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GETTERS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;      
        ;; Player           ;; Enemy 1             ;Bullets
            ;; 0000 Standing    ;; 0000 Standing        ;; 0000 Normal
            ;; 0001 Death       ;; 0001 Death           ;; 0001 Destroy
            ;; 0010 Shooting    ;; 0010 Death damage 
            ;; 0011 Down        ;; 0011 Walking
            ;; 0100 Up          
        ;;
    ;/////////////////////////////////////
    ;// FUNCTION
    ;// Input:
    ;//   B - Animation Status
    ;//   HL - Ponter to Animation Array
    ;// Return:
    ;//   IX - Pointer to Animation
    ;/////////////////////////////////////
    man_animations_getAnim::
        ;ld hl, #_animations_array ;; [3]
        ld a, b     ;; A = Animation Status
        add a       ;; A *= 2

        add_hl_a

        ld a, (hl)
        inc hl
        ld h, (hl)
        ld l, a
        push hl ;;/ IX = HL
        pop ix  ;;\ 
        ret
    ;/////////////////////////////////////
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PLAYER ANIMATIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    man_anim_player_standing::
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_wizard_00
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_wizard_01
        .db 0
        .dw man_anim_player_standing
    ;;
    man_anim_player_up::
        .db ANIM_FRAME_SPEED_160
        .dw #_spr_wizard_05
        .db ANIM_FRAME_SPEED_160
        .dw #_spr_wizard_06
        .db ANIM_FRAME_SPEED_160
        .dw #_spr_wizard_07
        .db 0
        .dw man_anim_player_up
    ;;
    man_anim_player_down::
        .db ANIM_FRAME_SPEED_160
        .dw #_spr_wizard_02
        .db ANIM_FRAME_SPEED_160
        .dw #_spr_wizard_03
        .db ANIM_FRAME_SPEED_160
        .dw #_spr_wizard_04
        .db 0
        .dw man_anim_player_down
    ;;

    man_anim_player_shooting::
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_wizard_08
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_wizard_09
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_wizard_10
        .db ANIM_FRAME_SPEED_50
        .dw #_spr_wizard_11
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_wizard_12
        .db 0
        .dw man_anim_player_shooting
    ;;

    man_anim_player_death::
        .db ANIM_FRAME_SPEED_150
        .dw #_spr_wizard_13
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_wizard_14
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_wizard_15
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_wizard_16
        .db ANIM_FRAME_SPEED_150
        .dw #_spr_wizard_17
        .db ANIM_FRAME_SPEED_150
        .dw #_spr_wizard_18
        .db ANIM_FRAME_SPEED_150
        .dw #_spr_wizard_19
        .db ANIM_FRAME_SPEED_1000
        .dw #_spr_wizard_20
        .db 0
        .dw man_anim_player_death
    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ENEMIES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    man_anim_enemy_death::
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_00
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_01
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_02
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_03
        .db ANIM_FRAME_SPEED_250
        .dw #_spr_enemy_death_04
        .db 0
        .dw man_anim_enemy_death
    ;;
    man_anim_enemy_death_damage::
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_05
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_06
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_07
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_08
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_09
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_10
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_11
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_12
        .db ANIM_FRAME_SPEED_120
        .dw #_spr_enemy_death_13
        .db 0
        .dw man_anim_enemy_death
    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ENEMY_FIRE ANIMATIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    man_anim_enemy_fire_standing::
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_fire_0
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_fire_1
        .db 0
        .dw man_anim_enemy_fire_standing
    ;;

    man_anim_enemy_fire_walking::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_fire_2
        .db ANIM_FRAME_SPEED_400
        .dw #_spr_enemy_fire_3
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_fire_4
        .db 0
        .dw man_anim_enemy_fire_walking
    ;;

    ;man_anim_enemy_fire_shooting::
        ;.dw man_anim_enemy_fire_shooting
    ;;

    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ENEMY_WATER ANIMATIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    man_anim_enemy_water_standing::
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_water_0
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_water_1
        .db 0
        .dw man_anim_enemy_water_standing
    ;;

    man_anim_enemy_water_walking::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_water_2
        .db ANIM_FRAME_SPEED_400
        .dw #_spr_enemy_water_3
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_water_4
        .db 0
        .dw man_anim_enemy_water_walking
    ;;
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ENEMY_ELECTRIC ANIMATIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    man_anim_enemy_electric_standing::
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_electric_0
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_electric_1
        .db 0
        .dw man_anim_enemy_electric_standing
    ;;

    man_anim_enemy_electric_walking::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_electric_2
        .db ANIM_FRAME_SPEED_400
        .dw #_spr_enemy_electric_3
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_electric_4
        .db 0
        .dw man_anim_enemy_electric_walking
    ;;
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ENEMY_LETTER ANIMATIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    man_anim_enemy_letter_standing::
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_letter_0
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_letter_1
        .db 0
        .dw man_anim_enemy_letter_standing
    ;;

    man_anim_enemy_letter_walking::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_letter_2
        .db ANIM_FRAME_SPEED_400
        .dw #_spr_enemy_letter_3
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_letter_4
        .db 0
        .dw man_anim_enemy_letter_walking
    ;;
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ENEMY_MIXED ANIMATIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    man_anim_enemy_mixed_standing::
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_mixed_0
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_mixed_1
        .db 0
        .dw man_anim_enemy_mixed_standing
    ;;

    man_anim_enemy_mixed_walking::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_mixed_2
        .db ANIM_FRAME_SPEED_400
        .dw #_spr_enemy_mixed_3
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_mixed_4
        .db 0
        .dw man_anim_enemy_mixed_walking
    ;;
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ENEMY_HEART ANIMATIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    man_anim_enemy_heart_standing::
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_heart_0
        .db ANIM_FRAME_SPEED_500
        .dw #_spr_enemy_heart_1
        .db 0
        .dw man_anim_enemy_heart_standing
    ;;

    man_anim_enemy_heart_walking::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_heart_2
        .db ANIM_FRAME_SPEED_400
        .dw #_spr_enemy_heart_3
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_enemy_heart_4
        .db 0
        .dw man_anim_enemy_heart_walking
    ;;
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SPELLS ANIMATIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    man_anim_spell_fire::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_fire_0
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_fire_1
        .db 0
        .dw man_anim_spell_fire
    ;;
    man_anim_spell_fire_destroy::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_fire_2
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_fire_3
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_fire_4
        .db 0
        .dw man_anim_spell_fire
    ;;

    man_anim_spell_water::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_water_0
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_water_1
        .db 0
        .dw man_anim_spell_water
    ;;
    man_anim_spell_water_destroy::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_water_2
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_water_3
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_water_4
        .db 0
        .dw man_anim_spell_water
    ;;


    man_anim_spell_electric::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_electric_0
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_electric_1
        .db 0
        .dw man_anim_spell_electric
    ;;
    man_anim_spell_electric_destroy::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_electric_2
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_electric_3
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_electric_4
        .db 0
        .dw man_anim_spell_electric
    ;;

    man_anim_spell_letter::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_letter_0
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_letter_1
        .db 0
        .dw man_anim_spell_letter
    ;;
    man_anim_spell_letter_destroy::
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_letter_2
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_letter_3
        .db ANIM_FRAME_SPEED_200
        .dw #_spr_spell_letter_4
        .db 0
        .dw man_anim_spell_letter
    ;;
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;