;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------

.include "./entity.h.s"
.include "../cpctelera_includes.h.s"

.area _DATA

MAX_ENTITIES           == 50
RESERVED_ENTITY_BYTES  == ENTITY_SIZE_WITH_PTR * MAX_ENTITIES

first_entity:
.dw #0x0000
next_free_entity:
.dw #entities_linked_stack
entities_linked_stack::
.ds RESERVED_ENTITY_BYTES

.area _CODE

entity_man_init::
   ;; Reset everything
   ld hl, #0x0000
   ld (first_entity), hl

   ld hl, #entities_linked_stack
   ld a, #0
   ld bc, #RESERVED_ENTITY_BYTES
   ex de, hl
   call cpct_memset_asm

   ld hl, #entities_linked_stack
   ld (next_free_entity), hl

   ld a,  #MAX_ENTITIES - 1
   ld bc, #ENTITY_SIZE_WITH_PTR
   ld ix, #entities_linked_stack                          ;; ix points to the first entity of the linked stack
   ld hl, #entities_linked_stack + #ENTITY_SIZE_WITH_PTR  ;; hl points to the second entity of the linked stack

   init_linked_stack_loop:
      ld ENTITY_NEXT_L(ix), l
      ld ENTITY_NEXT_H(ix), h

      add hl, bc
      add ix, bc
      dec a
      jr nz, init_linked_stack_loop
ret

;/////////////////////////////////////
;// Input:
;// Output: IX -> Entidad, DE -> Entidad sin el next entity
;// Destroys: A, HL, BC, DE, IX
;/////////////////////////////////////
entity_man_create_entity::
   ;; #next_free_entity pointer points to the next free entity
   ld ix, (#next_free_entity) ;; ix points to the new entity created
   ld h, ENTITY_NEXT_H(ix)
   ld l, ENTITY_NEXT_L(ix)
   ld (#next_free_entity), hl ;; #next_free_entity points to the new next free entity
   
   ;; The new entity points to the pointer stored into #first_entity
   ld hl, (#first_entity)
   ld ENTITY_NEXT_L(ix), l
   ld ENTITY_NEXT_H(ix), h
   
   ;; #first_entity and de point to the new entity
   push ix
   pop de
   ld (#first_entity), de
   ;; de points to the first data element of the entity
   inc de
   inc de
   ;; Load the entity size so a ldir call can be used after calling this procedure
   ld bc, #ENTITY_SIZE
ret



entity_man_for_all::
   ld hl, (#first_entity)

   entity_man_for_all_loop:
      ;; Check if the memory address is NULL (== 0) and exit the loop if it is
      ld a, h
      or a, l
      ret z

      ;;Load the entity in iy
      push hl
      pop iy

      ;; Call the function ptr previously set
      entity_man_for_all_function_ptr::
      call 0x0000

      ;; Store the next entity address in hl
      ld l, ENTITY_NEXT_L(iy)
      ld h, ENTITY_NEXT_H(iy)
      jr entity_man_for_all_loop
ret

entity_man_for_all_matching::
   ;; Reserve one bit for the mask parameter in the stack
   ld  ix, #-1
   add ix, sp
   ld  (ix), a
   ld  sp, ix

   ld hl, (#first_entity)

   entity_man_for_all_matching_loop:
      ;; Check if the memory address is NULL (== 0) and exit the loop if it is
      ld a, h
      or a, l
      jr z, entity_man_for_all_matching_exit_loop

      ;;Load the entity in iy
      push hl
      pop iy

      ;; Check if the entity mask matches the mask parameter
      ld  ix, #0
      add ix, sp
      ld  a, (ix)
      and ENTITY_CMPS(iy)
      cp  (ix)
      jr nz, entity_man_for_all_matching_next_entity

      ;; Call the function ptr previously set
      entity_man_for_all_matching_function_ptr::
      call 0x0000

   entity_man_for_all_matching_next_entity:
      ;; Store the next entity address in hl
      ld l, ENTITY_NEXT_L(iy)
      ld h, ENTITY_NEXT_H(iy)
      jr entity_man_for_all_matching_loop

   entity_man_for_all_matching_exit_loop:
      ;; Free the memory allocated to store the mask
      ld  ix, #1
      add ix, sp
      ld  sp, ix
ret

.include "../macros.h.s"
entity_man_for_all_pairs_matching::
   ;; Reserve one bit for the mask parameter in the stack
   ld  ix, #-1
   add ix, sp
   ld  (ix), a
   ld  sp, ix

   ld hl, (#first_entity)

   ;; while entity1.type != INVALID
   entity_man_for_all_pairs_matching_outer_loop:
      ;; Check if the memory address is NULL (== 0) and exit the loop if it is
      ld a, h
      or a, l
      jr z, entity_man_for_all_matching_exit_outer_loop

      ;;Load the entity in iy
      push hl
      pop iy

      ;; Check if the entity mask matches the mask parameter
      ld  hl, #0
      add hl, sp
      ld  a, (hl)
      and ENTITY_CMPS(iy)
      cp  (hl)
      jr nz, entity_man_for_all_pairs_matching_outer_loop_next_entity

      ld l, ENTITY_NEXT_L(iy)
      ld h, ENTITY_NEXT_H(iy)
      
      ;; while entity2.type != INVALID
      entity_man_for_all_pairs_matching_inner_loop:
         ;; Check if the memory address is NULL (== 0) and exit the loop if it is
         ld a, h
         or a, l
         jr z, entity_man_for_all_pairs_matching_outer_loop_next_entity

         ;; Load the entity in ix
         push hl
         pop ix

         ;; Check if the entity matches the mask parameter
         ld  hl, #0
         add hl, sp
         ld  a, (hl)
         and ENTITY_CMPS(ix)
         cp  (hl)
         jr nz, entity_man_for_all_pairs_matching_inner_loop_next_entity

         ;; Call the function ptr previously set
         entity_man_for_all_pairs_matching_function_ptr::
         call #0x0000

      entity_man_for_all_pairs_matching_inner_loop_next_entity:
         ;; Store the next entity address in hl
         ld l, ENTITY_NEXT_L(ix)
         ld h, ENTITY_NEXT_H(ix)
         jr entity_man_for_all_pairs_matching_inner_loop

   entity_man_for_all_pairs_matching_outer_loop_next_entity:
      ;; Store the next entity address in hl
      ld l, ENTITY_NEXT_L(iy)
      ld h, ENTITY_NEXT_H(iy)
      jr entity_man_for_all_pairs_matching_outer_loop

   entity_man_for_all_matching_exit_outer_loop:
      ;; Free the memory allocated to store the mask
      ld  ix, #1
      add ix, sp
      ld  sp, ix
ret

entity_man_update::
   ld ix, #first_entity
   ld hl, (#first_entity)

   ;; while entity_ptr != NULL
   entity_man_update_loop:
      ;; Check if the memory address is NULL (== 0) and exit the loop if it is
      ld a, h
      or a, l
      ret z

      ;; Load the entity in iy
      push hl
      pop iy

      ;; Store the next entity address in hl (It is need later)
      ld l, ENTITY_NEXT_L(iy)
      ld h, ENTITY_NEXT_H(iy)

      ;; Check if the entity is dead
      ld a, ENTITY_STATE(iy)
      and a, #ENTITY_STATE_DEAD 
      jr z, entity_man_update_next  ;;If entity->state & dead == 0 -> Test the next entity

      ;; Remove the entity
      ;; Set the previous entity next pointer attribute to point to next entity
      ld ENTITY_NEXT_L(ix), l  
      ld ENTITY_NEXT_H(ix), h    

      ;; Set the current entity that has been removed to point to the next_free_entity
      ld de, (#next_free_entity)
      ld ENTITY_NEXT_L(iy), e
      ld ENTITY_NEXT_H(iy), d
      ;; Set the next_free_entity value to be the current entity that has been removed
      ld (#next_free_entity), iy

      jr entity_man_update_loop

   entity_man_update_next:
      ;; Store the current entity in ix so in the next iteration it points to the previous entity
      push iy
      pop ix

      jr entity_man_update_loop
ret