;;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------

.include "./entity.h.s"
.include "./animations.h.s"
.include "../system/ai.h.s"
.include "../system/update.h.s"
.include "../sprites/sprites.h.s"
.include "./game.h.s"
.include "../constants.h.s"

PLAYER_BASE_COMPONENTS = COMPONENT_RENDER | COMPONENT_COLLIDER | COMPONENT_ANIMATED | COMPONENT_UPDATE
ENEMY_BASE_COMPONENTS  = COMPONENT_RENDER | COMPONENT_COLLIDER | COMPONENT_ANIMATED ;; | COMPONENT_UPDATE
BULLET_BASE_COMPONENTS = COMPONENT_RENDER | COMPONENT_COLLIDER | COMPONENT_UPDATE   | COMPONENT_ANIMATED
ENTITY_START_PREV_PTR1 = #0xC138
ENTITY_START_PREV_PTR2 = #0xC538
player_tmpl::
    .db ENTITY_TYPE_PLAYER                                      ;;ENTITY_TYPE               
    .db PLAYER_BASE_COMPONENTS                                  ;;ENTITY_CMPS               
    .db ENTITY_STATE_DEFAULT                                    ;;ENTITY_STATE              
    .db PLAYER_START_POS_X                                      ;;ENTITY_X                  
    .db 16                                                      ;;ENTITY_Y                  
    .db _SPR_PLAYER_W                                           ;;ENTITY_W                  
    .db _SPR_PLAYER_H                                           ;;ENTITY_H                  
    .db COLLISION_TYPE_PLAYER                                   ;;ENTITY_COLLISION_TYPE     
    .db COLLISION_TYPE_ENEMY_BULLET                             ;;ENTITY_COLLIDES_AGAINST   
    .dw _spr_wizard_00                                          ;;ENTITY_SPRITE_L, ENTITY_SPRITE_H        
    .dw ENTITY_START_PREV_PTR1                                  ;;ENTITY_PREV_PTR1_L, ENTITY_PREV_PTR1_H
    .dw ENTITY_START_PREV_PTR2                                  ;;ENTITY_PREV_PTR2_L, ENTITY_PREV_PTR2_H
    .dw man_anim_player_standing                                ;;ENTITY_ANIM_L, ENTITY_ANIM_H
    .dw _animations_array_player                                ;;ENTITY_ANIM_ARRAY_PTR_L, ENTITY_ANIM_ARRAY_PTR_H   
    .db ANIM_ENTITIES_FIRST_FRAME_COUNTER                       ;;ENTITY_ANIM_COUNTER       
    .db 0                                                       ;;ENTITY_ANIM_STATUS        
    .db 0                                                       ;;ENTITY_ANIM_PREV_STATUS   
    .dw sys_update_behaviour_player                             ;;ENTITY_UPDATE_BEHAVIOUR_L, ENTITY_UPDATE_BEHAVIOUR_H
    .db 0                                                       ;;ENTITY_UPDATE_COUNTER     
    .db 0                                                       ;;ENTITY_LINE_X   
    .db 0                                                       ;;ENTITY_LINE_Y   
    .db 0                                                       ;;ENTITY_LIFE  
    .db 0                                                       ;;ENTITY_VELOCITY | ENTITY_DEAD_PTR_L
    .db 0                                                       ;;ENTITY_HIT_PTR_H 

player_red_bullet_tmpl::        
    .db ENTITY_TYPE_BULLET                                      ;;ENTITY_TYPE               
    .db BULLET_BASE_COMPONENTS                                  ;;ENTITY_CMPS               
    .db ENTITY_STATE_DEFAULT                                    ;;ENTITY_STATE              
    .db PLAYER_START_POS_X - _SPR_SPELL_W                       ;;ENTITY_X                  
    .db 16                                                      ;;ENTITY_Y                  
    .db _SPR_SPELL_W                                            ;;ENTITY_W                  
    .db _SPR_SPELL_H                                            ;;ENTITY_H                  
    .db COLLISION_TYPE_PLAYER_BULLET                            ;;ENTITY_COLLISION_TYPE     
    .db COLLISION_TYPE_ENEMY_BULLET | COLLISION_TYPE_ENEMY      ;;ENTITY_COLLIDES_AGAINST   
    .dw _spr_spell_fire_0                                       ;;ENTITY_SPRITE_L, ENTITY_SPRITE_H        
    .dw ENTITY_START_PREV_PTR1                                  ;;ENTITY_PREV_PTR1_L, ENTITY_PREV_PTR1_H
    .dw ENTITY_START_PREV_PTR2                                  ;;ENTITY_PREV_PTR2_L, ENTITY_PREV_PTR2_H
    .dw man_anim_spell_fire                                     ;;ENTITY_ANIM_L, ENTITY_ANIM_H
    .dw _animations_array_spell_fire                            ;;ENTITY_ANIM_ARRAY_PTR_L, ENTITY_ANIM_ARRAY_PTR_H   
    .db ANIM_FRAME_SPEED_200                                    ;;ENTITY_ANIM_COUNTER       
    .db ANIM_SPELL_SHOOTING                                     ;;ENTITY_ANIM_STATUS        
    .db ANIM_SPELL_SHOOTING                                     ;;ENTITY_ANIM_PREV_STATUS
    .dw sys_update_behaviour_player_bullet                      ;;ENTITY_UPDATE_BEHAVIOUR_L, ENTITY_UPDATE_BEHAVIOUR_H
    .db 0                                                       ;;ENTITY_UPDATE_COUNTER     
    .db -1                                                      ;;ENTITY_BULLET_VX         | ENTITY_LINE_X   
    .db BULLET_RED_COLOR_MASK                                   ;;ENTITY_BULLET_COLOR_MASK | ENTITY_LINE_Y   
    .db 0                                                       ;;ENTITY_LIFE  
    .db 0                                                       ;;ENTITY_VELOCITY | ENTITY_DEAD_PTR_L
    .db 0                                                       ;;ENTITY_HIT_PTR_H

player_cyan_bullet_tmpl::
    .db ENTITY_TYPE_BULLET                                      ;;ENTITY_TYPE               
    .db BULLET_BASE_COMPONENTS                                  ;;ENTITY_CMPS               
    .db ENTITY_STATE_DEFAULT                                    ;;ENTITY_STATE              
    .db PLAYER_START_POS_X - _SPR_SPELL_W                       ;;ENTITY_X                  
    .db 16                                                      ;;ENTITY_Y                  
    .db _SPR_SPELL_W                                            ;;ENTITY_W                  
    .db _SPR_SPELL_H                                            ;;ENTITY_H                  
    .db COLLISION_TYPE_PLAYER_BULLET                            ;;ENTITY_COLLISION_TYPE     
    .db COLLISION_TYPE_ENEMY_BULLET | COLLISION_TYPE_ENEMY      ;;ENTITY_COLLIDES_AGAINST   
    .dw _spr_spell_water_0                                      ;;ENTITY_SPRITE_L, ENTITY_SPRITE_H        
    .dw ENTITY_START_PREV_PTR1                                  ;;ENTITY_PREV_PTR1_L, ENTITY_PREV_PTR1_H
    .dw ENTITY_START_PREV_PTR2                                  ;;ENTITY_PREV_PTR2_L, ENTITY_PREV_PTR2_H
    .dw man_anim_spell_water                                    ;;ENTITY_ANIM_L, ENTITY_ANIM_H
    .dw _animations_array_spell_water                           ;;ENTITY_ANIM_ARRAY_PTR_L, ENTITY_ANIM_ARRAY_PTR_H   
    .db ANIM_FRAME_SPEED_200                                    ;;ENTITY_ANIM_COUNTER       
    .db ANIM_SPELL_SHOOTING                                     ;;ENTITY_ANIM_STATUS        
    .db ANIM_SPELL_SHOOTING                                     ;;ENTITY_ANIM_PREV_STATUS   
    .dw sys_update_behaviour_player_bullet                      ;;ENTITY_UPDATE_BEHAVIOUR_L, ENTITY_UPDATE_BEHAVIOUR_H
    .db 0                                                       ;;ENTITY_UPDATE_COUNTER     
    .db -1                                                      ;;ENTITY_BULLET_VX         | ENTITY_LINE_X   
    .db BULLET_CYAN_COLOR_MASK                                  ;;ENTITY_BULLET_COLOR_MASK | ENTITY_LINE_Y   
    .db 0                                                       ;;ENTITY_LIFE  
    .db 0                                                       ;;ENTITY_VELOCITY | ENTITY_DEAD_PTR_L
    .db 0                                                       ;;ENTITY_HIT_PTR_H


player_yellow_bullet_tmpl::
    .db ENTITY_TYPE_BULLET                                      ;;ENTITY_TYPE               
    .db BULLET_BASE_COMPONENTS                                  ;;ENTITY_CMPS               
    .db ENTITY_STATE_DEFAULT                                    ;;ENTITY_STATE              
    .db PLAYER_START_POS_X - _SPR_SPELL_W                       ;;ENTITY_X                  
    .db 16                                                      ;;ENTITY_Y                  
    .db _SPR_SPELL_W                                            ;;ENTITY_W                  
    .db _SPR_SPELL_H                                            ;;ENTITY_H                  
    .db COLLISION_TYPE_PLAYER_BULLET                            ;;ENTITY_COLLISION_TYPE     
    .db COLLISION_TYPE_ENEMY_BULLET | COLLISION_TYPE_ENEMY      ;;ENTITY_COLLIDES_AGAINST   
    .dw _spr_spell_electric_0                                   ;;ENTITY_SPRITE_L, ENTITY_SPRITE_H        
    .dw ENTITY_START_PREV_PTR1                                  ;;ENTITY_PREV_PTR1_L, ENTITY_PREV_PTR1_H
    .dw ENTITY_START_PREV_PTR2                                  ;;ENTITY_PREV_PTR2_L, ENTITY_PREV_PTR2_H
    .dw man_anim_spell_electric                                 ;;ENTITY_ANIM_L, ENTITY_ANIM_H
    .dw _animations_array_spell_electric                        ;;ENTITY_ANIM_ARRAY_PTR_L, ENTITY_ANIM_ARRAY_PTR_H   
    .db ANIM_FRAME_SPEED_200                                    ;;ENTITY_ANIM_COUNTER       
    .db ANIM_SPELL_SHOOTING                                     ;;ENTITY_ANIM_STATUS        
    .db ANIM_SPELL_SHOOTING                                     ;;ENTITY_ANIM_PREV_STATUS   
    .dw sys_update_behaviour_player_bullet                      ;;ENTITY_UPDATE_BEHAVIOUR_L, ENTITY_UPDATE_BEHAVIOUR_H
    .db 0                                                       ;;ENTITY_UPDATE_COUNTER     
    .db -1                                                      ;;ENTITY_BULLET_VX         | ENTITY_LINE_X   
    .db BULLET_YELLOW_COLOR_MASK                                ;;ENTITY_BULLET_COLOR_MASK | ENTITY_LINE_Y   
    .db 0                                                       ;;ENTITY_LIFE  
    .db 0                                                       ;;ENTITY_VELOCITY | ENTITY_DEAD_PTR_L
    .db 0                                                       ;;ENTITY_HIT_PTR_H

enemy_letter_bullet_tmpl::
    .db ENTITY_TYPE_BULLET                                      ;;ENTITY_TYPE               
    .db BULLET_BASE_COMPONENTS                                  ;;ENTITY_CMPS               
    .db ENTITY_STATE_DEFAULT                                    ;;ENTITY_STATE              
    .db 0                                                       ;;ENTITY_X                  
    .db 0                                                       ;;ENTITY_Y                  
    .db _SPR_SPELL_W                                            ;;ENTITY_W                  
    .db _SPR_SPELL_H                                            ;;ENTITY_H                  
    .db COLLISION_TYPE_ENEMY_BULLET                             ;;ENTITY_COLLISION_TYPE     
    .db COLLISION_TYPE_PLAYER_BULLET | COLLISION_TYPE_PLAYER    ;;ENTITY_COLLIDES_AGAINST   
    .dw _spr_spell_letter_0                                     ;;ENTITY_SPRITE_L, ENTITY_SPRITE_H        
    .dw ENTITY_START_PREV_PTR1                                  ;;ENTITY_PREV_PTR1_L, ENTITY_PREV_PTR1_H
    .dw ENTITY_START_PREV_PTR2                                  ;;ENTITY_PREV_PTR2_L, ENTITY_PREV_PTR2_H
    .dw man_anim_spell_letter                                   ;;ENTITY_ANIM_L, ENTITY_ANIM_H
    .dw _animations_array_spell_letter                          ;;ENTITY_ANIM_ARRAY_PTR_L, ENTITY_ANIM_ARRAY_PTR_H   
    .db ANIM_FRAME_SPEED_200                                    ;;ENTITY_ANIM_COUNTER       
    .db ANIM_SPELL_SHOOTING                                     ;;ENTITY_ANIM_STATUS        
    .db ANIM_SPELL_SHOOTING                                     ;;ENTITY_ANIM_PREV_STATUS   
    .dw sys_update_behaviour_enemy_bullet                       ;;ENTITY_UPDATE_BEHAVIOUR_L, ENTITY_UPDATE_BEHAVIOUR_H
    .db 0                                                       ;;ENTITY_UPDATE_COUNTER     
    .db 1                                                       ;;ENTITY_BULLET_VX         | ENTITY_LINE_X   
    .db 0                                                       ;;ENTITY_BULLET_COLOR_MASK | ENTITY_LINE_Y   
    .db 0                                                       ;;ENTITY_LIFE  
    .db 0                                                       ;;ENTITY_VELOCITY | ENTITY_DEAD_PTR_L
    .db 0                                                       ;;ENTITY_HIT_PTR_H

enemy_fire_tmpl::         
    .db ENTITY_TYPE_ENEMY                                       ;;ENTITY_TYPE               
    .db ENEMY_BASE_COMPONENTS                                   ;;ENTITY_CMPS               
    .db ENTITY_STATE_DEFAULT                                    ;;ENTITY_STATE              
    .db PLAYER_START_POS_X                                      ;;ENTITY_X                  
    .db 16                                                      ;;ENTITY_Y                 
    .db _SPR_ENEMY_W                                            ;;ENTITY_W                  
    .db _SPR_ENEMY_H                                            ;;ENTITY_H                  
    .db COLLISION_TYPE_ENEMY                                    ;;ENTITY_COLLISION_TYPE     
    .db COLLISION_TYPE_PLAYER_BULLET                            ;;ENTITY_COLLIDES_AGAINST   
    .dw _spr_enemy_fire_0                                       ;;ENTITY_SPRITE_L, ENTITY_SPRITE_H        
    .dw ENTITY_START_PREV_PTR1                                  ;;ENTITY_PREV_PTR1_L, ENTITY_PREV_PTR1_H
    .dw ENTITY_START_PREV_PTR2                                  ;;ENTITY_PREV_PTR2_L, ENTITY_PREV_PTR2_H
    .dw man_anim_enemy_fire_standing                            ;;ENTITY_ANIM_L, ENTITY_ANIM_H
    .dw _animations_array_enemy_fire                            ;;ENTITY_ANIM_ARRAY_PTR_L, ENTITY_ANIM_ARRAY_PTR_H   
    .db ANIM_ENTITIES_FIRST_FRAME_COUNTER                       ;;ENTITY_ANIM_COUNTER       
    .db 0                                                       ;;ENTITY_ANIM_STATUS        
    .db 0                                                       ;;ENTITY_ANIM_PREV_STATUS   
    .dw 0x0000                                                  ;;ENTITY_UPDATE_BEHAVIOUR_L, ENTITY_UPDATE_BEHAVIOUR_H
    .db 0                                                       ;;ENTITY_UPDATE_COUNTER     
    .db 0                                                       ;;ENTITY_LINE_X   
    .db 0                                                       ;;ENTITY_LINE_Y   
    .db 0b11111111                                              ;;ENTITY_LIFE      
    .db 0                                                       ;;ENTITY_VELOCITY | ENTITY_DEAD_PTR_L
    .db 0                                                       ;;ENTITY_HIT_PTR_H

enemy_water_tmpl::         
    .db ENTITY_TYPE_ENEMY                                       ;;ENTITY_TYPE               
    .db ENEMY_BASE_COMPONENTS                                   ;;ENTITY_CMPS               
    .db ENTITY_STATE_DEFAULT                                    ;;ENTITY_STATE              
    .db PLAYER_START_POS_X                                      ;;ENTITY_X                  
    .db 16                                                      ;;ENTITY_Y                 
    .db _SPR_ENEMY_W                                            ;;ENTITY_W                  
    .db _SPR_ENEMY_H                                            ;;ENTITY_H                  
    .db COLLISION_TYPE_ENEMY                                    ;;ENTITY_COLLISION_TYPE     
    .db COLLISION_TYPE_PLAYER_BULLET                            ;;ENTITY_COLLIDES_AGAINST   
    .dw _spr_enemy_water_0                                      ;;ENTITY_SPRITE_L, ENTITY_SPRITE_H        
    .dw ENTITY_START_PREV_PTR1                                  ;;ENTITY_PREV_PTR1_L, ENTITY_PREV_PTR1_H
    .dw ENTITY_START_PREV_PTR2                                  ;;ENTITY_PREV_PTR2_L, ENTITY_PREV_PTR2_H
    .dw man_anim_enemy_water_standing                           ;;ENTITY_ANIM_L, ENTITY_ANIM_H
    .dw _animations_array_enemy_water                           ;;ENTITY_ANIM_ARRAY_PTR_L, ENTITY_ANIM_ARRAY_PTR_H   
    .db ANIM_ENTITIES_FIRST_FRAME_COUNTER                       ;;ENTITY_ANIM_COUNTER       
    .db 0                                                       ;;ENTITY_ANIM_STATUS        
    .db 0                                                       ;;ENTITY_ANIM_PREV_STATUS   
    .dw 0x0000                                                  ;;ENTITY_UPDATE_BEHAVIOUR_L, ENTITY_UPDATE_BEHAVIOUR_H
    .db 0                                                       ;;ENTITY_UPDATE_COUNTER     
    .db 0                                                       ;;ENTITY_LINE_X   
    .db 0                                                       ;;ENTITY_LINE_Y
    .db 0b10101010                                              ;;ENTITY_LIFE  
    .db 0                                                       ;;ENTITY_VELOCITY | ENTITY_DEAD_PTR_L
    .db 0                                                       ;;ENTITY_HIT_PTR_H    


enemy_electric_tmpl::         
    .db ENTITY_TYPE_ENEMY                                       ;;ENTITY_TYPE               
    .db ENEMY_BASE_COMPONENTS                                   ;;ENTITY_CMPS               
    .db ENTITY_STATE_DEFAULT                                    ;;ENTITY_STATE              
    .db PLAYER_START_POS_X                                      ;;ENTITY_X                  
    .db 16                                                      ;;ENTITY_Y                 
    .db _SPR_ENEMY_W                                            ;;ENTITY_W                  
    .db _SPR_ENEMY_H                                            ;;ENTITY_H                  
    .db COLLISION_TYPE_ENEMY                                    ;;ENTITY_COLLISION_TYPE     
    .db COLLISION_TYPE_PLAYER_BULLET                            ;;ENTITY_COLLIDES_AGAINST   
    .dw _spr_enemy_electric_0                                   ;;ENTITY_SPRITE_L, ENTITY_SPRITE_H        
    .dw ENTITY_START_PREV_PTR1                                  ;;ENTITY_PREV_PTR1_L, ENTITY_PREV_PTR1_H
    .dw ENTITY_START_PREV_PTR2                                  ;;ENTITY_PREV_PTR2_L, ENTITY_PREV_PTR2_H
    .dw man_anim_enemy_electric_standing                        ;;ENTITY_ANIM_L, ENTITY_ANIM_H
    .dw _animations_array_enemy_electric                        ;;ENTITY_ANIM_ARRAY_PTR_L, ENTITY_ANIM_ARRAY_PTR_H   
    .db ANIM_ENTITIES_FIRST_FRAME_COUNTER                       ;;ENTITY_ANIM_COUNTER       
    .db 0                                                       ;;ENTITY_ANIM_STATUS        
    .db 0                                                       ;;ENTITY_ANIM_PREV_STATUS   
    .dw 0x0000                                                  ;;ENTITY_UPDATE_BEHAVIOUR_L, ENTITY_UPDATE_BEHAVIOUR_H
    .db 0                                                       ;;ENTITY_UPDATE_COUNTER     
    .db 0                                                       ;;ENTITY_LINE_X   
    .db 0                                                       ;;ENTITY_LINE_Y
    .db 0b01010101                                              ;;ENTITY_LIFE  
    .db 0                                                       ;;ENTITY_VELOCITY | ENTITY_DEAD_PTR_L
    .db 0                                                       ;;ENTITY_HIT_PTR_H  

enemy_mixed_tmpl::         
    .db ENTITY_TYPE_ENEMY                                       ;;ENTITY_TYPE               
    .db ENEMY_BASE_COMPONENTS                                   ;;ENTITY_CMPS               
    .db ENTITY_STATE_RANDOM_HEALTHBAR                           ;;ENTITY_STATE              
    .db PLAYER_START_POS_X                                      ;;ENTITY_X                  
    .db 16                                                      ;;ENTITY_Y                 
    .db _SPR_ENEMY_W                                            ;;ENTITY_W                  
    .db _SPR_ENEMY_H                                            ;;ENTITY_H                  
    .db COLLISION_TYPE_ENEMY                                    ;;ENTITY_COLLISION_TYPE     
    .db COLLISION_TYPE_PLAYER_BULLET                            ;;ENTITY_COLLIDES_AGAINST   
    .dw _spr_enemy_mixed_0                                      ;;ENTITY_SPRITE_L, ENTITY_SPRITE_H        
    .dw ENTITY_START_PREV_PTR1                                  ;;ENTITY_PREV_PTR1_L, ENTITY_PREV_PTR1_H
    .dw ENTITY_START_PREV_PTR2                                  ;;ENTITY_PREV_PTR2_L, ENTITY_PREV_PTR2_H
    .dw man_anim_enemy_mixed_standing                           ;;ENTITY_ANIM_L, ENTITY_ANIM_H
    .dw _animations_array_enemy_mixed                           ;;ENTITY_ANIM_ARRAY_PTR_L, ENTITY_ANIM_ARRAY_PTR_H   
    .db ANIM_ENTITIES_FIRST_FRAME_COUNTER                       ;;ENTITY_ANIM_COUNTER       
    .db 0                                                       ;;ENTITY_ANIM_STATUS        
    .db 0                                                       ;;ENTITY_ANIM_PREV_STATUS   
    .dw 0x0000                                                  ;;ENTITY_UPDATE_BEHAVIOUR_L, ENTITY_UPDATE_BEHAVIOUR_H
    .db 0                                                       ;;ENTITY_UPDATE_COUNTER     
    .db 0                                                       ;;ENTITY_LINE_X   
    .db 0                                                       ;;ENTITY_LINE_Y
    .db 0                                                       ;;ENTITY_LIFE  
    .db 0                                                       ;;ENTITY_VELOCITY | ENTITY_DEAD_PTR_L
    .db 0                                                       ;;ENTITY_HIT_PTR_H  

enemy_letter_tmpl::         
    .db ENTITY_TYPE_ENEMY                                       ;;ENTITY_TYPE               
    .db ENEMY_BASE_COMPONENTS | COMPONENT_UPDATE               ;;ENTITY_CMPS               
    .db ENTITY_STATE_RANDOM_HEALTHBAR                           ;;ENTITY_STATE              
    .db PLAYER_START_POS_X                                      ;;ENTITY_X                  
    .db 16                                                      ;;ENTITY_Y                 
    .db _SPR_ENEMY_W                                            ;;ENTITY_W                  
    .db _SPR_ENEMY_H                                            ;;ENTITY_H                  
    .db COLLISION_TYPE_ENEMY                                    ;;ENTITY_COLLISION_TYPE     
    .db COLLISION_TYPE_PLAYER_BULLET                            ;;ENTITY_COLLIDES_AGAINST   
    .dw _spr_enemy_letter_0                                     ;;ENTITY_SPRITE_L, ENTITY_SPRITE_H        
    .dw ENTITY_START_PREV_PTR1                                  ;;ENTITY_PREV_PTR1_L, ENTITY_PREV_PTR1_H
    .dw ENTITY_START_PREV_PTR2                                  ;;ENTITY_PREV_PTR2_L, ENTITY_PREV_PTR2_H
    .dw man_anim_enemy_letter_standing                          ;;ENTITY_ANIM_L, ENTITY_ANIM_H
    .dw _animations_array_enemy_letter                          ;;ENTITY_ANIM_ARRAY_PTR_L, ENTITY_ANIM_ARRAY_PTR_H   
    .db ANIM_ENTITIES_FIRST_FRAME_COUNTER                       ;;ENTITY_ANIM_COUNTER       
    .db 0                                                       ;;ENTITY_ANIM_STATUS        
    .db 0                                                       ;;ENTITY_ANIM_PREV_STATUS   
    .dw sys_update_behaviour_enemy_letter                       ;;ENTITY_UPDATE_BEHAVIOUR_L, ENTITY_UPDATE_BEHAVIOUR_H
    .db 0                                                       ;;ENTITY_UPDATE_COUNTER     
    .db 0                                                       ;;ENTITY_LINE_X   
    .db 0                                                       ;;ENTITY_LINE_Y
    .db 0                                                       ;;ENTITY_LIFE    
    .db 0                                                       ;;ENTITY_VELOCITY | ENTITY_DEAD_PTR_L
    .db 0                                                       ;;ENTITY_HIT_PTR_H

enemy_heart_tmpl::         
    .db ENTITY_TYPE_ENEMY                                       ;;ENTITY_TYPE               
    .db ENEMY_BASE_COMPONENTS | COMPONENT_DEAD_PTR              ;;ENTITY_CMPS               
    .db ENTITY_STATE_RANDOM_HEALTHBAR                           ;;ENTITY_STATE              
    .db PLAYER_START_POS_X                                      ;;ENTITY_X                  
    .db 16                                                      ;;ENTITY_Y                 
    .db _SPR_ENEMY_W                                            ;;ENTITY_W                  
    .db _SPR_ENEMY_H                                            ;;ENTITY_H                  
    .db COLLISION_TYPE_ENEMY                                    ;;ENTITY_COLLISION_TYPE     
    .db COLLISION_TYPE_PLAYER_BULLET                            ;;ENTITY_COLLIDES_AGAINST   
    .dw _spr_enemy_heart_0                                      ;;ENTITY_SPRITE_L, ENTITY_SPRITE_H        
    .dw ENTITY_START_PREV_PTR1                                  ;;ENTITY_PREV_PTR1_L, ENTITY_PREV_PTR1_H
    .dw ENTITY_START_PREV_PTR2                                  ;;ENTITY_PREV_PTR2_L, ENTITY_PREV_PTR2_H
    .dw man_anim_enemy_heart_standing                           ;;ENTITY_ANIM_L, ENTITY_ANIM_H
    .dw _animations_array_enemy_heart                           ;;ENTITY_ANIM_ARRAY_PTR_L, ENTITY_ANIM_ARRAY_PTR_H   
    .db ANIM_ENTITIES_FIRST_FRAME_COUNTER                       ;;ENTITY_ANIM_COUNTER       
    .db 0                                                       ;;ENTITY_ANIM_STATUS        
    .db 0                                                       ;;ENTITY_ANIM_PREV_STATUS   
    .dw 0x0000                                                  ;;ENTITY_UPDATE_BEHAVIOUR_L, ENTITY_UPDATE_BEHAVIOUR_H
    .db 0                                                       ;;ENTITY_UPDATE_COUNTER     
    .db 0                                                       ;;ENTITY_LINE_X   
    .db 0                                                       ;;ENTITY_LINE_Y
    .db 0                                                       ;;ENTITY_LIFE    
    .dw game_add_player_health                                  ;;ENTITY_DEAD_PTR_L, ENTITY_HIT_PTR_H
