;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------

.include "../cpctelera_includes.h.s"
.include "../system/lanes.h.s"
.include "../macros.h.s"

.area _DATA

MENU_INTERRUPTIONS_COUNTER_INIT_VALUE = 12 - 2
GAME_INTERRUPTIONS_COUNTER_INIT_VALUE = 12 - 5
FPS_COUNTER_INIT_VALUE = 1 ;; 1 -> 6 Ints per frame = 50fps -> 12 Ints per frame = 25fps 

interruptions_counter: .db MENU_INTERRUPTIONS_COUNTER_INIT_VALUE
fps_counter: .db FPS_COUNTER_INIT_VALUE

.area _CODE

interruptions_init::
    im 1

    call cpct_disableFirmware_asm
ret

interruptions_wait_sync::
    interruptions_wait_sync_loop:
        ld a, (#fps_counter)
        or a
        jr nz, interruptions_wait_sync_loop
    
    ld a, #FPS_COUNTER_INIT_VALUE
    ld (#fps_counter), a
ret

interruptions_menu_int_handler::
    di
    
    ;; Self-modifying jump
    interruptions_menu_int_handler_jump:
        jp  interruptions_menu_int_handler_update_counter
        
    ;; ###### COUNTER ######
    interruptions_menu_int_handler_update_counter:
        push af
        push hl
        
        ld  a, (#interruptions_counter)
        dec a
        jr z, interruptions_menu_int_handler_next_int_type
            ;; Store the new counter value and jump to the end
            ld (#interruptions_counter), a
            jr interruptions_menu_int_handler_end_counter

        interruptions_menu_int_handler_next_int_type:
            ;; Change the interruption jump to jump to the keyboard label and reset the counter
            ld hl, #interruptions_menu_int_handler_scan_keyboard
            ld (#interruptions_menu_int_handler_jump + #1), hl
            ld a, #MENU_INTERRUPTIONS_COUNTER_INIT_VALUE
            ld (#interruptions_counter), a

            jr interruptions_menu_int_handler_end_counter

    ;; ###### KEYBOARD ######
    interruptions_menu_int_handler_scan_keyboard:
        push af
        push hl
        push bc
        push de

        call cpct_scanKeyboard_if_asm

        ;; Change the interruption jump to jump to the music label
        ld hl, #interruptions_menu_int_handler_play_music
        ld (#interruptions_menu_int_handler_jump + #1), hl
        jr interruptions_menu_int_handler_end_keyboard

    ;; ###### MUSIC ######
    interruptions_menu_int_handler_play_music:
        push af
        push hl
        push bc
        push de
        push ix
        push iy

        ;; Call the play music and safe the corresponding register
        call cpct_akp_musicPlay_asm
        ;; Change the interruption jump to jump to the update counter label
        ld hl, #interruptions_menu_int_handler_update_counter
        ld (#interruptions_menu_int_handler_jump + #1), hl

        pop iy
        pop ix

    interruptions_menu_int_handler_end_keyboard:
        pop de
        pop bc

    interruptions_menu_int_handler_end_counter:
        pop hl
        pop af

    ei
    reti


interruptions_game_int_handler::
    di
    
    ;; Self-modifying jump
    interruptions_game_int_handler_jump:
        jp  interruptions_game_int_handler_update_counter

    ;; ###### COUNTER ######
    interruptions_game_int_handler_update_counter:
        push af
        push hl
        
        ld  a, (#interruptions_counter)
        dec a
        jr z, interruptions_game_int_handler_next_int_type
            ;; Store the new counter value and jump to the end
            ld (#interruptions_counter), a
            jr interruptions_game_int_handler_end_counter

        interruptions_game_int_handler_next_int_type:
            ;; Change the interruption jump to jump to the fps counter label and reset the counter
            ld hl, #interruptions_game_int_handler_scan_keyboard
            ld (#interruptions_game_int_handler_jump + #1), hl
            ld a, #GAME_INTERRUPTIONS_COUNTER_INIT_VALUE
            ld (#interruptions_counter), a

            jr interruptions_game_int_handler_end_counter

    ;; ###### KEYBOARD ######
    interruptions_game_int_handler_scan_keyboard:
        push af
        push hl
        push bc
        push de

        call cpct_scanKeyboard_if_asm

        ;; Change the interruption jump to jump to the music label
        ld hl, #interruptions_game_int_handler_play_music
        ld (#interruptions_game_int_handler_jump + #1), hl
        jr interruptions_game_int_handler_end_keyboard

    ;; ###### MUSIC ######
    interruptions_game_int_handler_play_music:
        push af
        push hl
        push bc
        push de
        push ix
        push iy

        ;; Call the play music and safe the corresponding register
        call cpct_akp_musicPlay_asm
        ;; Change the interruption jump to jump to the lanes label
        ld hl, #interruptions_game_int_handler_update_lanes
        ld (#interruptions_game_int_handler_jump + #1), hl
        jr interruptions_game_int_handler_end_lanes

    ;; ###### LANES ######
    interruptions_game_int_handler_update_lanes:
        push af
        push hl
        push bc
        push de
        push ix
        push iy

        call sys_lanes_update_timers

        ;; Change the interruption jump to jump to the counter label
        ld hl, #interruptions_game_int_handler_update_fps_counter
        ld (#interruptions_game_int_handler_jump + #1), hl
        jr interruptions_game_int_handler_end_lanes

    interruptions_game_int_handler_update_fps_counter:
        push af
        push hl
        
        ld  a, (#fps_counter)
        or a
        jr z, interruptions_game_int_handler_fps_next_int_type
            dec a
            ld (#fps_counter), a
        interruptions_game_int_handler_fps_next_int_type:
            ;; Change the interruption jump to jump to the keyboard label
            ld hl, #interruptions_game_int_handler_update_counter
            ld (#interruptions_game_int_handler_jump + #1), hl
            jr interruptions_game_int_handler_end_counter

    interruptions_game_int_handler_end_lanes:
        pop iy
        pop ix

    interruptions_game_int_handler_end_keyboard:
        pop de
        pop bc

    interruptions_game_int_handler_end_counter:
        pop hl
        pop af

    ei
    reti

;/////////////////////////////////////
;// Input:
;//     HL - Interruption handler
;// Destroys: A
;/////////////////////////////////////
interruptions_set_interruption_handler::
    call cpct_waitVSYNC_asm
    halt
    halt
    call cpct_waitVSYNC_asm
    
    di

    ;; Reset the function jumps
    ex de, hl
    ld hl, #interruptions_menu_int_handler_update_counter
    ld (#interruptions_menu_int_handler_jump + #1), hl

    ld hl, #interruptions_game_int_handler_update_counter
    ld (#interruptions_game_int_handler_jump + #1), hl
    ex de, hl

    ld a, #GAME_INTERRUPTIONS_COUNTER_INIT_VALUE
    ld (#interruptions_counter), a

    ;; Load the jp instruction
    ld a, #0xC3
    ld (#0x0038), a
    ;; Load the jp address
    ld (#0x0039), hl

    ei
ret

interruptions_disable_interruptions::
    call cpct_disableFirmware_asm
ret