;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------

.include "cpctelera.h.s"
.include "../system/render.h.s"
.include "../system/level.h.s"
.include "../system/lanes.h.s"
.include "../cpctelera_includes.h.s"
.include "../constants.h.s"
.include "../render_cpctelera32x16.h.s"
.include "./interruptions.h.s"
.include "./screen_manager.h.s"
.include "../macros.h.s"

.globl _screen_load_level_z_end

STRING_LEVEL_SIZE     = 5
STRING_COMPLETED_SIZE = 9
STRING_GAME_OVER_SIZE = 9
STRING_CONTINUE_SIZE  = 9
STRING_YES_SIZE       = 6
STRING_NO_SIZE        = 5
STRING_YOU_WIN_SIZE   = 7

STRING_LEVEL_CENTER_X    = ((SCREEN_WIDTH_IN_CHARS / 2) * STRING_CHAR_WIDTH - (STRING_LEVEL_SIZE / 2) * STRING_CHAR_WIDTH) - (STRING_CHAR_WIDTH / 2)
STRING_COMPLETED_CENTER_X = ((SCREEN_WIDTH_IN_CHARS / 2) * STRING_CHAR_WIDTH - (STRING_COMPLETED_SIZE / 2) * STRING_CHAR_WIDTH) - (STRING_CHAR_WIDTH / 2)
STRING_GAME_OVER_CENTER_X = ((SCREEN_WIDTH_IN_CHARS / 2) * STRING_CHAR_WIDTH - (STRING_GAME_OVER_SIZE / 2) * STRING_CHAR_WIDTH) - (STRING_CHAR_WIDTH / 2)
STRING_CONTINUE_CENTER_X  = ((SCREEN_WIDTH_IN_CHARS / 2) * STRING_CHAR_WIDTH - (STRING_CONTINUE_SIZE / 2) * STRING_CHAR_WIDTH) - (STRING_CHAR_WIDTH / 2)
STRING_YES_CENTER_X       = ((SCREEN_WIDTH_IN_CHARS / 2) * STRING_CHAR_WIDTH - (STRING_YES_SIZE / 2) * STRING_CHAR_WIDTH) - (STRING_CHAR_WIDTH)
STRING_NO_CENTER_X        = ((SCREEN_WIDTH_IN_CHARS / 2) * STRING_CHAR_WIDTH - (STRING_NO_SIZE / 2) * STRING_CHAR_WIDTH) - (STRING_CHAR_WIDTH / 2)
STRING_YOU_WIN_CENTER_X   = ((SCREEN_WIDTH_IN_CHARS / 2) * STRING_CHAR_WIDTH - (STRING_YOU_WIN_SIZE / 2) * STRING_CHAR_WIDTH) - (STRING_CHAR_WIDTH / 2)
STRING_CENTER_Y           = ((SCREEN_HEIGHT_IN_CHARS / 2) * STRING_CHAR_HEIGHT - (STRING_CHAR_HEIGHT / 2))

string_completed:
    .asciz "COMPLETED"
string_level: 
    .asciz "LEVEL"
string_game_over:
    .asciz "GAME OVER"

string_continue:
    .asciz "CONTINUE?"
string_yes:
    .asciz "[1]YES"
string_no:
    .asciz "[2]NO"
string_you_win:
    .asciz "YOU WIN"

load_level_screen_common_init::
    call cpct_akp_stop_asm
    call interruptions_disable_interruptions
    call sys_render_reset_double_buffer
    

    ld hl, #_screen_load_level_z_end
    ld de, #0xFFFF
    call cpct_zx7b_decrunch_s_asm
ret

load_level_screen_next_level_init::
    call load_level_screen_common_init

    call level_load_next_level
ret

load_level_screen_current_level_init::
    call load_level_screen_common_init

    call level_load_current_level
ret

wait::
   wait_loop:

   ld b, #2
   call cpct_waitHalts_asm
   call cpct_waitVSYNC_asm
   
   dec d
   jr nz, wait_loop
ret

draw_string:
    ld de,   #BUFFER_PTR_1
    call getScreenPtr_32x16
    call cpct_drawStringM0_32x16
ret

draw_string_delayed:
    draw_string_delayed_loop:
        push af

        ld a, 0(iy)
        or a
        jr z, draw_string_delayed_exit_loop

        pop af
        push af
        push hl
        ld d, a
        call wait

        push iy
        ld e, 0(iy)
        call cpct_drawCharM0_32x16
        pop iy

        pop hl
        pop af
        
        ld bc, #STRING_CHAR_WIDTH
        add hl, bc
        inc iy
    jr draw_string_delayed_loop
    draw_string_delayed_exit_loop:
        pop af
ret

draw_level_completed_string:
    ld h, #3
    ld l, #13
    call cpct_setDrawCharM0_32x16

    ld de, #BUFFER_PTR_1
    ld c, #STRING_LEVEL_CENTER_X
    ld b, #STRING_CENTER_Y - (STRING_CHAR_HEIGHT / 2)
    call getScreenPtr_32x16

    ld a, #6
    ld iy, #string_level
    call draw_string_delayed

    ld de, #BUFFER_PTR_1
    ld c, #STRING_COMPLETED_CENTER_X
    ld b, #STRING_CENTER_Y + (STRING_CHAR_HEIGHT / 2)
    call getScreenPtr_32x16

    ld a, #6
    ld iy, #string_completed
    call draw_string_delayed
ret

draw_level_string:
    ld h, #3
    ld l, #1
    call cpct_setDrawCharM0_32x16

    ld de, #BUFFER_PTR_1
    ld c, #STRING_LEVEL_CENTER_X - STRING_CHAR_WIDTH - (STRING_CHAR_WIDTH / 2)
    ld b, #STRING_CENTER_Y
    call getScreenPtr_32x16

    ld a, #6
    ld iy, #string_level
    call draw_string_delayed

    ld a, #4
    ld bc, #STRING_CHAR_WIDTH
    add hl, bc
    next_level_screen_update_level_num_loop:
        push af
        push hl

        ld c, #STRING_CHAR_WIDTH * 2
        ld b, #STRING_CHAR_HEIGHT
        ld a, #0xCC
        ex de, hl
        call drawSolidBox_32x16

        ld d, #12
        call wait

        pop hl
        push hl

        ld a, (#level_generator_current_num_level_left)
        add #'0'
        ld e, a
        call cpct_drawCharM0_32x16

        pop hl
        push hl
        
        ld bc, #STRING_CHAR_WIDTH
        add hl, bc

        ld a, (#level_generator_current_num_level_right)
        add #'0'
        ld e, a
        call cpct_drawCharM0_32x16

        ld d, #12
        call wait

        pop hl
        pop af
        dec a
    jr nz, next_level_screen_update_level_num_loop
ret

load_level_screen_current_level_update::
    call draw_level_string
    
    call screen_man_to_game_screen

load_level_screen_game_over_update::
    ld h, #3
    ld l, #4
    call cpct_setDrawCharM0_32x16

    ld de, #BUFFER_PTR_1
    ld c, #STRING_GAME_OVER_CENTER_X
    ld b, #STRING_CENTER_Y - (STRING_CHAR_HEIGHT * 4)
    call getScreenPtr_32x16

    ld a, #6
    ld iy, #string_game_over
    call draw_string_delayed

    ld d, #12
    call wait

    ld h, #3
    ld l, #13
    call cpct_setDrawCharM0_32x16
    
    ld c,    #STRING_CONTINUE_CENTER_X
    ld b,    #STRING_CENTER_Y - (STRING_CHAR_HEIGHT * 1)
    ld iy,   #string_continue
    call draw_string

    ld c,    #STRING_YES_CENTER_X + (STRING_CHAR_WIDTH / 2)
    ld b,    #STRING_CENTER_Y + (STRING_CHAR_HEIGHT)
    ld iy,   #string_yes
    call draw_string

    ld c,    #STRING_NO_CENTER_X - (STRING_CHAR_WIDTH)
    ld b,    #STRING_CENTER_Y + (STRING_CHAR_HEIGHT * 3)
    ld iy,   #string_no
    call draw_string

load_level_screen_game_over_update_loop:
    call cpct_scanKeyboard_f_asm
        ld hl, #Key_1
        call cpct_isKeyPressed_asm
        call nz, screen_man_to_current_level_screen

        ld hl, #Key_2
        call cpct_isKeyPressed_asm
        call nz, screen_man_to_menu_screen
    jr load_level_screen_game_over_update_loop

load_level_screen_victory_update::
    ld h, #3
    ld l, #8
    call cpct_setDrawCharM0_32x16

    ld de, #BUFFER_PTR_1
    ld c, #STRING_YOU_WIN_CENTER_X
    ld b, #STRING_CENTER_Y
    call getScreenPtr_32x16

    ld a, #6
    ld iy, #string_you_win
    call draw_string_delayed

    ld a, #4
    load_level_screen_victory_update_loop:
        push af

        ld d, #12
        call wait

        ld c,    #STRING_YOU_WIN_CENTER_X
        ld b,    #STRING_CENTER_Y
        ld de,   #BUFFER_PTR_1
        call getScreenPtr_32x16

        ld c, #STRING_YOU_WIN_SIZE * STRING_CHAR_WIDTH
        ld b, #STRING_CHAR_HEIGHT
        ld a, #0xCC
        ex de, hl
        call drawSolidBox_32x16

        ld d, #12
        call wait

        ld c,    #STRING_YOU_WIN_CENTER_X
        ld b,    #STRING_CENTER_Y
        ld iy,   #string_you_win
        call draw_string

        pop af
        dec a
        jr nz, load_level_screen_victory_update_loop


    ld d, #64
    call wait
    call screen_man_to_menu_screen

load_level_screen_next_level_update::
    call draw_level_completed_string

    ld d, #32
    call wait

    ld de, #BUFFER_PTR_1
    ld c, #STRING_COMPLETED_CENTER_X
    ld b, #STRING_CENTER_Y - (STRING_CHAR_HEIGHT / 2)
    call getScreenPtr_32x16

    ld c, #STRING_CHAR_WIDTH * STRING_COMPLETED_SIZE
    ld b, #STRING_CHAR_HEIGHT * 2
    ld a, #0xCC
    ex de, hl
    call drawSolidBox_32x16

    call draw_level_string
    call screen_man_to_game_screen