;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------

.module menu

.include "cpctelera.h.s"
.include "../render_cpctelera32x16.h.s"
.include "../cpctelera_includes.h.s"
.include "../music/music.h.s"
.include "../manager/interruptions.h.s"
.include "../screens/screen_title_z.h.s"
.include "../macros.h.s"
.include "../constants.h.s"
.include "../system/render.h.s"
.include "./screen_manager.h.s"
.include "../system/level.h.s"

.area _CODE

CHAR_SIZE_X = 4
CHAR_SIZE_Y = 8
CHAR_SEP_4_LANES_Y = 6
CHAR_INIT_Y_LANE = 7
CHAR_LANES_HEIGHT = CHAR_SIZE_Y*8
CHAR_4_LANES_0_WITHOUT_SEP = CHAR_SIZE_Y*CHAR_INIT_Y_LANE
CHAR_4_LANES_0 = CHAR_SIZE_Y*CHAR_INIT_Y_LANE+CHAR_SEP_4_LANES_Y
CHAR_4_LANES_1 = CHAR_4_LANES_0+CHAR_SIZE_Y+CHAR_SEP_4_LANES_Y
CHAR_4_LANES_2 = CHAR_4_LANES_1+CHAR_SIZE_Y+CHAR_SEP_4_LANES_Y
CHAR_4_LANES_3 = CHAR_4_LANES_2+CHAR_SIZE_Y+CHAR_SEP_4_LANES_Y

CHAR_SEP_CREDITS_LANES_Y_0 = 7
CHAR_SEP_CREDITS_LANES_Y_1 = 3
CHAR_CREDITS_0 = CHAR_4_LANES_0_WITHOUT_SEP
CHAR_CREDITS_1 = CHAR_CREDITS_0+CHAR_SIZE_Y+CHAR_SEP_CREDITS_LANES_Y_1
CHAR_CREDITS_2 = CHAR_CREDITS_1+CHAR_SIZE_Y+CHAR_SEP_CREDITS_LANES_Y_1
CHAR_CREDITS_3 = CHAR_CREDITS_2+CHAR_SIZE_Y+CHAR_SEP_CREDITS_LANES_Y_1
CHAR_CREDITS_4 = CHAR_CREDITS_3+CHAR_SIZE_Y+CHAR_SEP_CREDITS_LANES_Y_0



;; START MENU
   string_start_menu_play: .asciz "[1]PLAY"
   string_start_menu_controls: .asciz "[2]CONTROLS"
   START_MENU_CONTROLS_X_SEP = (5*CHAR_SIZE_X)+(CHAR_SIZE_X/2)
   START_MENU_CONTROLS_X = (SCREEN_WIDTH_IN_BYTES/2)-START_MENU_CONTROLS_X_SEP
   string_start_menu_credits: .asciz "[3]CREDITS"
   string_start_menu_exit: .asciz "[4]EXIT"
   START_MENU_MAX_X = START_MENU_CONTROLS_X
;;

;; CONTROLS MENU
   .globl _spr_controls
   SPR_CONTROLS_W = 56
   SPR_CONTROLS_H = 42
   SPR_CONTROLS_X = (SCREEN_WIDTH_IN_BYTES/2)-(SPR_CONTROLS_W/2)
;;

;; CREDITS MENU
   .globl _spr_credits_ts
   SPR_CREDITS_TS_W = 16
   SPR_CREDITS_TS_H = 8
   SPR_CREDITS_TS_X = (SCREEN_WIDTH_IN_BYTES/2)-(SPR_CREDITS_TS_W/2)

   string_credits_menu_luis: .asciz "LUIS MORATA"
   CREDITS_MENU_LUIS_X_SEP = (5*CHAR_SIZE_X)+(CHAR_SIZE_X/2)
   CREDITS_MENU_LUIS_X = (SCREEN_WIDTH_IN_BYTES/2)-CREDITS_MENU_LUIS_X_SEP


   string_credits_menu_lucia: .asciz "LUCIA OLIVA"
   CREDITS_MENU_LUCIA_X_SEP = (5*CHAR_SIZE_X)+(CHAR_SIZE_X/2)
   CREDITS_MENU_LUCIA_X = (SCREEN_WIDTH_IN_BYTES/2)-CREDITS_MENU_LUCIA_X_SEP


   string_credits_menu_victor: .asciz "VICTOR TEBAR"
   CREDITS_MENU_VICTOR_X_SEP = (6*CHAR_SIZE_X)
   CREDITS_MENU_VICTOR_X = (SCREEN_WIDTH_IN_BYTES/2)-CREDITS_MENU_VICTOR_X_SEP
;;

string_menu_return: .asciz "[ESC]RETURN"
STRING_MENU_RETURN_X_SEP = (5*CHAR_SIZE_X)+(CHAR_SIZE_X/2)
STRING_MENU_RETURN_X = (SCREEN_WIDTH_IN_BYTES/2)-STRING_MENU_RETURN_X_SEP


;//////////////////////////
;// MENU DRAW STRING
;// Input
;// c  - Pos X
;// d  - Pos Y
;// ix - String
;//////////////////////////
man_menu_draw_string:
   ld de,   #0xC000
   call getScreenPtr_32x16
   call cpct_drawStringM0_32x16
ret

man_menu_clear_strings:
   ld de,   #0xC000
   ld c,    #3
   ld b,    #CHAR_4_LANES_0_WITHOUT_SEP
   call getScreenPtr_32x16
   ex de,   hl
   ld a,    #0
   ld c,    #SCREEN_WIDTH_IN_BYTES-6
   ld b,    #CHAR_LANES_HEIGHT
   call drawSolidBox_32x16
ret

man_menu_start_init::
   call sys_render_reset_double_buffer
   call level_reset

   ld h, #0
   ld l, #1
   call cpct_setDrawCharM0_32x16

   ld hl, #_screen_title_z_end
   ld de, #0xFFFF
   call cpct_zx7b_decrunch_s_asm

   ld de, #_song_menu
   call cpct_akp_musicInit_asm 
   ld  hl, #interruptions_menu_int_handler
   call interruptions_set_interruption_handler
ret

man_menu_start::

   call man_menu_clear_strings

   ; NO HACE FALTA CON LA NUEVA PALETA
   ;ld h, #1
   ;ld l, #0
   ;call cpct_setDrawCharM0_32x16


   
   ld c,    #START_MENU_MAX_X
   ld b,    #CHAR_4_LANES_0
   ld iy,   #string_start_menu_play
   call     man_menu_draw_string

   ld c,    #START_MENU_MAX_X
   ld b,    #CHAR_4_LANES_1
   ld iy,   #string_start_menu_controls
   call     man_menu_draw_string

   ld c,    #START_MENU_MAX_X
   ld b,    #CHAR_4_LANES_2
   ld iy,   #string_start_menu_credits
   call     man_menu_draw_string

   ld c,    #START_MENU_MAX_X
   ld b,    #CHAR_4_LANES_3
   ld iy,   #string_start_menu_exit
   call     man_menu_draw_string

   
   man_menu_start_loop:
   call cpct_getRandom_mxor_u8_asm

   ;Play
   ld hl, #Key_1
   call cpct_isKeyPressed_asm
   call nz, screen_man_to_current_level_screen

   ;Controls
   ld hl, #Key_2
   call cpct_isKeyPressed_asm
   jr z, Controls_NotPressed
   Controls_Pressed:
      jp man_menu_controls
   Controls_NotPressed:

   ;Credits
   ld hl, #Key_3
   call cpct_isKeyPressed_asm
   jr z, Credits_NotPressed
   Credits_Pressed:
      jp man_menu_credits
   Credits_NotPressed:

   ;Exit
   ld hl, #Key_4
   call cpct_isKeyPressed_asm
   jr z, EscStart_NotPressed
   EscStart_Pressed:
      ld a, #0
      or a
      call screen_man_to_exit
   EscStart_NotPressed:

   jr man_menu_start_loop

man_menu_controls:
   call man_menu_clear_strings

   ; NO HACE FALTA CON LA NUEVA PALETA
   ;ld h, #0
   ;ld l, #15
   ;call cpct_setDrawCharM0_32x16

   ld de,   #0xC000
   ld c,    #SPR_CONTROLS_X
   ld b,    #CHAR_4_LANES_0_WITHOUT_SEP
   call getScreenPtr_32x16
   ex de,   hl
   ld hl,   #_spr_controls
   ld c,    #SPR_CONTROLS_W
   ld b,    #SPR_CONTROLS_H
   call     drawSprite_32x16

   ld c,    #STRING_MENU_RETURN_X
   ld b,    #CHAR_4_LANES_3
   ld iy,   #string_menu_return
   call     man_menu_draw_string

   man_menu_controls_loop:

   ;Salir
   ld hl, #Key_Esc
   call cpct_isKeyPressed_asm
   jp nz, man_menu_start

   jr man_menu_controls_loop

man_menu_credits:
   call man_menu_clear_strings

   ; NO HACE FALTA CON LA NUEVA PALETA
   ;ld h, #0
   ;ld l, #15
   ;call cpct_setDrawCharM0_32x16
   
   ld de,   #0xC000
   ld c,    #SPR_CREDITS_TS_X
   ld b,    #CHAR_CREDITS_0
   call getScreenPtr_32x16
   ex de,   hl
   ld hl,   #_spr_credits_ts
   ld c,    #SPR_CREDITS_TS_W
   ld b,    #SPR_CREDITS_TS_H
   call     drawSprite_32x16

   ld c,    #CREDITS_MENU_LUIS_X
   ld b,    #CHAR_CREDITS_1
   ld iy,   #string_credits_menu_luis
   call     man_menu_draw_string

   ld c,    #CREDITS_MENU_LUCIA_X
   ld b,    #CHAR_CREDITS_2
   ld iy,   #string_credits_menu_lucia
   call     man_menu_draw_string

   ld c,    #CREDITS_MENU_VICTOR_X
   ld b,    #CHAR_CREDITS_3
   ld iy,   #string_credits_menu_victor
   call     man_menu_draw_string

   ld c,    #STRING_MENU_RETURN_X
   ld b,    #CHAR_CREDITS_4
   ld iy,   #string_menu_return
   call     man_menu_draw_string

   man_menu_credits_loop:

   ;Salir
   ld hl, #Key_Esc
   call cpct_isKeyPressed_asm
   jp nz, man_menu_start

   jr man_menu_credits_loop