;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------


.module sys_animations
.include "../manager/entity.h.s"
.include "../manager/entity_manager.h.s"
.include "../manager/animations.h.s"
.include "../macros.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PRIVATE FUNCTIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;/////////////////////////////////////
    ;// FUNCTION
    ;//   Behaviour mothership
    ;// Input:
    ;//   IY - Pointer to Entity
    ;//   IX - Pointer to Anim array
    ;// Return:
    ;//   
    ;/////////////////////////////////////
    sys_animations_update_one_entity:
        
        ;// change sprite when animcounter is 0

        call sys_animations_calculate_status    ;; continue or change animation
        ;; B = Animation Status
        ld a, ENTITY_ANIM_PREV_STATUS(iy)    ;; A = Current Anim Status
        cp b                            ;; A == B
        jr z, _sys_anim_continue        ;; True -> continue
    _sys_anim_new_animation:
        ;; ponemos estado
        ld ENTITY_ANIM_STATUS(iy), b        ;; New animation status
        ld ENTITY_ANIM_PREV_STATUS(iy), b    ;; A = Current Anim Status
        
        call man_animations_getAnim
        ;; IX = Anim Pointer
        ;; ponemos counter de la animacion en entity
        ld a, ANIM_ARRAY_COUNTER(ix)        ;;/
        ld ENTITY_ANIM_COUNTER(iy), a       ;;\ e-> anim_counter = HL = e->anim->time
        ;; change animation
        ld ENTITY_ANIM_H(iy), h    ;; 
        ld ENTITY_ANIM_L(iy), l    ;; Change
        ;; change sprite
        ld h, ANIM_ARRAY_SPRITE+1(ix)
        ld l, ANIM_ARRAY_SPRITE(ix)
        ld ENTITY_SPRITE_L(iy), l
        ld ENTITY_SPRITE_H(iy), h
    _sys_anim_continue:
        ;;; if (-- (e->animcounter) == 0)
            ;; anim_counter--
                ld a, ENTITY_ANIM_COUNTER(iy)  ;;/
                dec a                       ;;| e->anim_counter--
                ld ENTITY_ANIM_COUNTER(iy), a  ;;\
            ;; 
            or a                       ;; == 0      
            ret nz
            

            ;; @(e->animcounter == 0)@

                    
                ;; ++e->anim 
                    ld h, ENTITY_ANIM_H(iy)     ;;/
                    ld l, ENTITY_ANIM_L(iy)     ;;| HL = e->anim
                    ld bc, #ANIM_ARRAY_SIZE     ;; size of counter + pointer (3)
                    add hl, bc                  ;;\ e->anim++

                    push hl               ;; HL = e->anim->siguiente?
                    pop ix

                ;; (IX = next frame)
                ;;; if( e->anim->time == 0)
                    ld a,  ANIM_ARRAY_COUNTER(ix) ;; 
                    or a
                    jr nz, sys_animations_update_one_entity_load_next_sprite
                    ;; @(e->anim->time == 0)@{
                        
                        ;; e->anim = e->anim->val.next
                            
                            ld l, ANIM_ARRAY_VALUE(ix)
                            ld h, ANIM_ARRAY_VALUE+1(ix)

                            ld ENTITY_ANIM_L(iy), l    ;;/
                            ld ENTITY_ANIM_H(iy), h    ;;\ e->anim = HL = e->anim->val
                            
                            push hl
                            pop ix
                        ;;
                    ;;}
                    sys_animations_update_one_entity_load_next_sprite:
                ;;;}
                

                ;// Load next sprite from animation
                    ld a, ANIM_ARRAY_COUNTER(ix)        ;;/
                    ld ENTITY_ANIM_COUNTER(iy), a       ;;\ e-> anim_counter = HL = e->anim->time
                    
                    ld ENTITY_ANIM_H(iy), h    ;; 
                    ld ENTITY_ANIM_L(iy), l    ;; Change
                ;;
                ;;e->sprite = e->anim->val.sprite
                    
                    ld h, ANIM_ARRAY_SPRITE+1(ix)
                    ld l, ANIM_ARRAY_SPRITE(ix)
                    
                    ld ENTITY_SPRITE_L(iy), l    ;;/
                    ld ENTITY_SPRITE_H(iy), h    ;;\ e->sprite = HL = e->anim->val
                    
                ;;
                    
            ;;
        ;;;  
        ret
    ;/////////////////////////////////////

    ;/////////////////////////////////////
    ;// FUNCTION
    ;//   calculate Animation Status
    ;// Input:
    ;//   IY - Pointer to Entity
    ;// Return:
    ;//   B - Animation Status
    ;/////////////////////////////////////
    sys_animations_calculate_status:
        ld h, ENTITY_ANIM_ARRAY_PTR_H(iy)   ;;/ HL = PTR to Animation Array
        ld l, ENTITY_ANIM_ARRAY_PTR_L(iy)   ;;\

        ld b, ENTITY_ANIM_STATUS(iy)   ;; B = New Status
        ; ;;; if (entity is dead) {
        ;     ld a, ENTITY_STATE(iy)
        ;     and #ENTITY_STATE_DEAD
        ;     jr z, _not_dead
        ;         ;; @( dead = TRUE)@ ; B = 0001
        ;             set 0,b
        ;             ret
        ;         ;; }
        ;     _not_dead:
        ; ;;; }
        ; ;;; if ( entity == enemy) {
        ;     ld a, ENTITY_CMPS(iy)   ;;/ Check if entity is enemy
        ;     and #COMPONENT_AI       ;;\
        ;     jr z, _isplayer
        ;     ;; @( is enemy )@
        ;     ld a, ENTITY_STATE(iy)
        ;     and #ENTITY_STATE_MOVING
        ;     jr z, _vx_enemy_zero
        ;         ;; @( STATE_MOVING )@; B = 0010
        ;             set 1,b 
        ;             ret
        ;         ;;
        ;     _vx_enemy_zero:
        ;         ret
        ; ;;; } else {
        ; _isplayer:
        ;     ld a, ENTITY_VY(iy)
        ;     or a
        ;     jr z, _vy_player_zero
        ;     _vy_player_nonzero:
        ;         jp m, _vy_player_negative
        ;     _vy_player_positive:
        ;         ;; @( vy > 0 )@ moving down
        ;             set 0,b ;;/ B = 0011
        ;             set 1,b ;;\
        ;         ;;
        ;     _vy_player_negative:
        ;         ;; @( vy < 0 )@ Moving up
        ;             set 2,b ;; B = 0100
        ;         ;;
        ;     _vy_player_zero:
        ; ;;; }
        ret
    ;/////////////////////////////////////
        ;; Player           ;; Enemy 1
        ;; 0000 Standing    ;; 0000 Standing
        ;; 0001 Death       ;; 0001 Death
        ;; 0010 Shooting    ;; 0010 Walking 
        ;; 0011 Down        
        ;; 0100 Up
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PUBLIC FUNCTIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;/////////////////////////////////////
    ;// FUNCTION
    ;//   Behaviour mothership
    ;// Input:
    ;//   IY - Pointer to Entity
    ;// Return:
    ;//   
    ;/////////////////////////////////////
    sys_animations_update::
        SET_CALL_INSTRUCTION_ADDRESS #entity_man_for_all_matching_function_ptr, #sys_animations_update_one_entity
        ld a, #COMPONENT_ANIMATED
        call entity_man_for_all_matching
        
        ret
    ;/////////////////////////////////////
      
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      