;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------



.include "../manager/entity.h.s"
.include "../manager/entity_manager.h.s"
.include "../manager/game.h.s"
.include "../manager/animations.h.s"
.include "../macros.h.s"

player_collisions::                  ;; Index 0
    .dw 0x0000                          ;; Player vs Player
    .dw 0x0000                          ;; Player vs Enemy
    .dw sys_collison_player_vs_bullet   ;; Player vs Bullet
enemy_collisions::                   ;; Index 1
    .dw 0x0000                          ;; Enemy vs Player
    .dw 0x0000                          ;; Enemy vs Enemy
    .dw sys_collision_enemy_vs_bullet   ;; Enemy vs Bullet
bullet_collisions::                  ;; Index 2
    .dw sys_collision_bullet_vs_player  ;; Bullet vs Player
    .dw sys_collision_bullet_vs_enemy   ;; Bullet vs Enemy
    .dw sys_collision_bullet_vs_bullet  ;; Bullet vs Bullet
collision_table::
    .dw player_collisions
    .dw enemy_collisions
    .dw bullet_collisions

;/////////////////////////////////////
;// Input: 
;//     IY: Player 
;//     IX: Bullet
;// Destroys: A, B
;/////////////////////////////////////
;; This function should NEVER be called because the player is always going to be the last entity in the stack
sys_collison_player_vs_bullet:
    ;; Swap the pointers
    push iy 
    push ix
    pop iy
    pop ix

    ;; This should work because "sys_collision_bullet_vs_player" jumps to the end of the sys_collision_update_entities function
    ;; that makes a ret, coming back to here to ret into the caller
    call sys_collision_bullet_vs_player

    ;; Swap the pointers back
    push iy 
    push ix
    pop iy
    pop ix
ret

;/////////////////////////////////////
;// Input: 
;//     IY: Enemy 
;//     IX: Bullet
;// Destroys: A, B
;/////////////////////////////////////
;; This function is less probable to be called as the bullets will be usually the first to be updated in the stack
sys_collision_enemy_vs_bullet:
    ;; Swap the pointers
    push iy 
    push ix
    pop iy
    pop ix

    ;; This should work because "sys_collision_bullet_vs_enemy" jumps to the end of the sys_collision_update_entities function
    ;; that makes a ret, coming back to here to ret into the caller
    call sys_collision_bullet_vs_enemy

    ;; Swap the pointers back
    push iy 
    push ix
    pop iy
    pop ix
ret


;/////////////////////////////////////
;// Input: 
;//     IY: Bullet 
;//     IX: Player
;// Destroys: A, B
;/////////////////////////////////////
sys_collision_bullet_vs_player:
    ;; Kill the bullet
    ld a, #ANIM_FRAME_SPEED_200 * 3
    call game_kill_bullet_iy

    call game_remove_player_health
jp sys_collision_update_entities_end


;/////////////////////////////////////
;// Input: 
;//     IY: Bullet 
;//     IX: Enemy
;// Destroys: A, B
;/////////////////////////////////////
sys_collision_bullet_vs_enemy::
    ;; kill the bullet
    ld a, #ANIM_FRAME_SPEED_200 * 3
    call game_kill_bullet_iy

    ;; Test if the color matches 
    ld a, ENTITY_LIFE(ix)
    and a, #BULLET_COLOR_MASK
    ld b, ENTITY_BULLET_COLOR_MASK(iy)
    cp b
    jr nz, sys_collision_bullet_vs_enemy_wrong_color
        ;; Remove the left most 2 bits
        ld a, ENTITY_LIFE(ix)
        sla a
        sla a
        ld ENTITY_LIFE(ix), a

        ;;if the entity is dead -> kill it
        or a
        call z, game_kill_enemy
        jp sys_collision_update_entities_end

    sys_collision_bullet_vs_enemy_wrong_color:
        ;; Check if there is space to add another color (Check if the 2 leftmost bits are 0)
        ld a, ENTITY_LIFE(ix)
        and #BULLET_FREE_SPACE_MASK
        jr z, sys_collision_bullet_vs_enemy_add_color
        jp sys_collision_update_entities_end

    sys_collision_bullet_vs_enemy_add_color:
        ;; Move the bits two positions to the right and add the new color
        ld a, ENTITY_LIFE(ix)
        srl a
        srl a
        or ENTITY_BULLET_COLOR_MASK(iy)
        ld ENTITY_LIFE(ix), a
jp sys_collision_update_entities_end

;/////////////////////////////////////
;// Input: 
;//     IY: Bullet 
;//     IX: Bullet
;// Destroys: A
;/////////////////////////////////////
sys_collision_bullet_vs_bullet:
    ;; Kill both bullets
    ld a, #ANIM_FRAME_SPEED_200 * 3
    call game_kill_bullet_iy

    push ix
    push iy
    push ix
    
    pop  iy
    call game_kill_bullet_iy

    pop iy
    pop ix
jr sys_collision_update_entities_end

sys_collision_update_entities::
    ;; Check if the entities can collide
    ld a, ENTITY_COLLIDES_AGAINST(ix)
    and ENTITY_COLLISION_TYPE(iy)
    ld b, a
    
    ld a, ENTITY_COLLIDES_AGAINST(iy)
    and ENTITY_COLLISION_TYPE(ix)
    or b
    ret z

    call sys_collision_check_collision
    jr nz, sys_collision_update_entities_end
        ;BRK
        ;; Get the function from the collision table

        ld hl, #collision_table
        ;; Calculate the first offset in the table
        ld b, #0
        ld c, ENTITY_TYPE(iy)
        sla c                   ;; Type *= 2
        add hl, bc

        ;; Set hl to point into the array of functions
        ld a, (hl)
        inc hl
        ld h, (hl)
        ld l, a

        ;; Set hl to point into the collision function in the array of functions
        ld c, ENTITY_TYPE(ix)
        sla c                   ;; Type *= 2
        add hl, bc

        ;; Set hl to point to the collision function
        ld a, (hl)
        inc hl
        ld h, (hl)
        ld l, a

        jp (hl)
sys_collision_update_entities_end:
ret

;; If collision -> z = 1
;; else -> z = 0
sys_collision_check_collision:
    ld a, #0

    ;; e1->x + e1->w < e2->x
    ld a, ENTITY_X(ix)
    add a, ENTITY_W(ix)
    cp ENTITY_X(iy)
    ret m

    ;; e1->x > e2->x + e2->w
    ld a, ENTITY_X(iy)
    add a, ENTITY_W(iy)
    cp ENTITY_X(ix)
    ret m

    ;; e1->y + e1->h < e2->y
    ld a, ENTITY_Y(ix)
    add a, ENTITY_H(ix)
    cp ENTITY_Y(iy)
    ret m

    ;; e1->y > e2->y + e2->h
    ld a, ENTITY_Y(iy)
    add a, ENTITY_H(iy)
    cp ENTITY_Y(ix)
    ret m

    ld a, #0
    or a
ret

sys_collisions_update::
    SET_CALL_INSTRUCTION_ADDRESS #entity_man_for_all_pairs_matching_function_ptr, #sys_collision_update_entities
    ld a, #COMPONENT_COLLIDER
    call entity_man_for_all_pairs_matching
ret