;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------


.include "../constants.h.s"
.include "../manager/game.h.s"
.include "../manager/entity.h.s"
.include "../sprites/sprites.h.s"
.include "../manager/animations.h.s"
.include "./render.h.s"
.include "./level.h.s"
.include "cpctelera.h.s"
.include "../render_cpctelera32x16.h.s"
.include "../macros.h.s"

.area _DATA

LANE_SLOT_ENTITY_L      == 0
LANE_SLOT_ENTITY_H      == 1
LANE_SLOT_POS_X         == 2
LANE_SLOT_POS_Y         == 3 ;; Size must be a power of 2 to be able to access it fast
LANE_SLOT_SIZE          == 4    

;; LINEAS
LANE_SLOTS_START_SLOT         == 0
LANE_SLOTS_END_SLOT           == (LANE_SLOTS_START_SLOT + LANE_NUM_SLOTS * LANE_SLOT_SIZE) - LANE_SLOT_SIZE
LANE_POS_X                    == LANE_SLOTS_END_SLOT + LANE_SLOT_SIZE
LANE_POS_Y                    == LANE_POS_X + 1
LANE_INDEX_Y                  == LANE_POS_X + 2
LANE_FRAME_COUNTER            == LANE_POS_X + 3
LANE_FRAME_COUNTER_INIT_VALUE == LANE_POS_X + 4
LANE_TIMER_VALUE              == LANE_POS_X + 5
LANE_STATE                    == LANE_POS_X + 6
LANE_UPDATE_FUNCTION_PTR_L    == LANE_POS_X + 7
LANE_UPDATE_FUNCTION_PTR_H    == LANE_POS_X + 8
LANE_SIZE                     == LANE_POS_X + 9
MAX_LANES                     == 3

lanes::
lane_1:
.ds LANE_SIZE
lane_2:
.ds LANE_SIZE
lane_3:
.ds LANE_SIZE

lanes_ptr_array::
.dw lane_1, lane_2, lane_3

.area _CODE


;/////////////////////////////////////
;// Input: 
;//     C: Indice linea 
;//     E: Indice entidad
;// Destroys: HL, BC, DE
;/////////////////////////////////////
sys_lanes_get_lane_entity_slot::
    sla c   ;; Lane index *= 2
    sla e   ;; 
    sla e   ;; Entity Index *= 4 (Entity Index *= LANE_SLOT_SIZE)
    
    ;; 
    ld b, #0
    ld d, #0

    ;; Set hl to point into the lanes ptr array
    ld hl, #lanes_ptr_array
    add hl, bc

    ;; Set hl to point to the start of the lane struct
    ld a, (hl)
    inc hl
    ld h, (hl)
    ld l, a

    ;; Set hl to point to the start of the slot 
    add hl, de
ret

;/////////////////////////////////////
;// Input: 
;//     C: Indice linea 
;// Destroys: A,BC,HL
;// 
;/////////////////////////////////////
sys_lanes_get_lane::
    sla c   ;; Lane index *= 2

    ld hl, #lanes_ptr_array
    ld b, #0
    add hl, bc

    ld a, (hl)
    inc hl
    ld h, (hl)
    ld l, a
ret

;/////////////////////////////////////
;// FUNCTION -> Sets the x positions of all the slots and lanes
;// Destroys: IX, BC, DE, A
;/////////////////////////////////////
sys_lanes_init::
    ld ix, #lane_1
    ld LANE_POS_X(ix), #LANE_SLOT_START_POS_X
    ld LANE_INDEX_Y(ix), #0
    call sys_lanes_set_lane_slots_x_position

    ld ix, #lane_2
    ld LANE_POS_X(ix), #LANE_SLOT_START_POS_X
    ld LANE_INDEX_Y(ix), #1
    call sys_lanes_set_lane_slots_x_position

    ld ix, #lane_3
    ld LANE_POS_X(ix), #LANE_SLOT_START_POS_X
    ld LANE_INDEX_Y(ix), #2
    call sys_lanes_set_lane_slots_x_position
ret

;/////////////////////////////////////
;// FUNCTION -> Sets the x positions of all the slots in a lane
;// Destroys: IX, BC, DE, A
;/////////////////////////////////////
sys_lanes_set_lane_slots_x_position:
    ld b, #0
    ld c, #LANE_SLOT_SIZE
    ld d, #LANE_NUM_SLOTS
    ld e, #LANE_SLOT_WIDTH + #LANE_SLOT_SEPARATION
    ld a, #LANE_SLOT_START_POS_X

    sys_lanes_set_lane_cells_x_position_loop:
        ;; 
        ld LANE_SLOT_POS_X(ix), a
        add a, e

        ;; Get the next slot
        add ix, bc
        dec d
        jr nz, sys_lanes_set_lane_cells_x_position_loop
ret

;/////////////////////////////////////
;// FUNCTION -> Sets the y positions of all the slots in a lane
;// Destroys: IX, BC, DE, A
;/////////////////////////////////////
sys_lanes_set_lane_slots_y_position:

ret

;/////////////////////////////////////
;// FUNCTION
;// Destroy: AF, BC, DE, HL, IY
;/////////////////////////////////////
sys_lanes_reset::
    ld ix, #lanes
    ld bc, #LANE_SIZE
    ld a, (#level_generator_num_lanes)
    ld d, a

    sys_lanes_reset_loop:
        ld a, LANE_FRAME_COUNTER_INIT_VALUE(ix)
        ld LANE_FRAME_COUNTER(ix), a
        ld LANE_TIMER_VALUE(ix), #LANE_TIMER_INIT_VALUE     
        ld LANE_STATE(ix), #LANE_STATE_UPDATING_COUNTER  

        call sys_lanes_reset_entities

        add ix, bc
        dec d
    jr nz, sys_lanes_reset_loop
ret

;/////////////////////////////////////
;// FUNCTION
;// Input: 
;//     IX -> Lane
;// Destroy: AF, E, HL
;/////////////////////////////////////
sys_lanes_reset_entities::
    ;; Store the lane entities in hl
    push ix
    pop hl

    xor a
    ld e, #LANE_NUM_SLOTS
    sys_lanes_reset_entities_loop:
        ;; Reset the pointer
        ld (hl), a
        inc hl
        ld (hl), a

        inc hl
        inc hl
        inc hl
        dec e
    jr nz, sys_lanes_reset_entities_loop
    
ret

;/////////////////////////////////////
;// FUNCTION
;// Input:
;//     IX - Player
;// Destroy: IY
;/////////////////////////////////////
sys_lanes_set_player_position::
    ld iy, #lane_1
    ld a, LANE_POS_Y(iy)
    ld ENTITY_Y(ix), a
ret

;/////////////////////////////////////
;// FUNCTION
;// Input:
;//     a - numero de lineas activas
;// Destroy: ix
;/////////////////////////////////////
sys_lanes_update_lanes_positions::
    ld a, (#level_generator_num_lanes)
sys_lanes_set_current_num_lanes_case_1_lanes:
    cp #1
    jr nz, sys_lanes_set_current_num_lanes_case_2_lanes
    ;; Case 1 lane
        ld ix, #lane_1
        ld LANE_POS_Y(ix), #LANE_1_LANES_START_POS_Y

        jr sys_lanes_set_current_num_lanes_end_switch

sys_lanes_set_current_num_lanes_case_2_lanes:
    cp #2
    jr nz, sys_lanes_set_current_num_lanes_case_3_lanes
    ;; Case 2 lanes
        ld ix, #lane_1
        ld LANE_POS_Y(ix), #LANE_2_LANES_START_POS_Y_1

        ld ix, #lane_2
        ld LANE_POS_Y(ix), #LANE_2_LANES_START_POS_Y_2

        jr sys_lanes_set_current_num_lanes_end_switch

sys_lanes_set_current_num_lanes_case_3_lanes:
    ;; Case 3 lanes
        ld ix, #lane_1
        ld LANE_POS_Y(ix), #LANE_3_LANES_START_POS_Y_1

        ld ix, #lane_2
        ld LANE_POS_Y(ix), #LANE_3_LANES_START_POS_Y_2
        
        ld ix, #lane_3
        ld LANE_POS_Y(ix), #LANE_3_LANES_START_POS_Y_3

sys_lanes_set_current_num_lanes_end_switch:
ret



;/////////////////////////////////////
;// Input:
;// Destroy: iy
;/////////////////////////////////////
sys_lanes_draw::
    ld iy, #lanes
    ld a,  (#level_generator_num_lanes)

    sys_lanes_draw_loop:
        push af
        call sys_lanes_draw_single_lane
        pop af

        ld bc, #LANE_SIZE
        add iy, bc
        dec a
        jr nz, sys_lanes_draw_loop
ret

;/////////////////////////////////////
;// Input: ix
;// Destroys: hl, bc, de, af
;/////////////////////////////////////
sys_lanes_draw_single_lane:
    call sys_lanes_draw_lane_timer

    ld b, #LANE_NUM_SLOTS - 1
    ld c, #LANE_SLOT_START_POS_X

    sys_lanes_draw_single_lane_loop:
        push bc

        ld de, #BUFFER_PTR_1
        ld b, LANE_POS_Y(iy)
        call getScreenPtr_32x16

        ex de, hl
        ld hl, #_spr_marker_enemy
        ld c, #_SPR_MARKER_ENEMY_W
        ld b, #_SPR_MARKER_ENEMY_H
        call drawSprite_32x16

        pop bc
        push bc

        ld de, #BUFFER_PTR_2
        ld b, LANE_POS_Y(iy)
        call getScreenPtr_32x16

        ex de, hl
        ld hl, #_spr_marker_enemy
        ld c, #_SPR_MARKER_ENEMY_W
        ld b, #_SPR_MARKER_ENEMY_H
        call drawSprite_32x16 

        pop bc
        ld a, #LANE_SLOT_SEPARATION + #LANE_SLOT_WIDTH
        add a, c
        ld c, a
        dec b
        jr nz, sys_lanes_draw_single_lane_loop

    ;; Draw player slot in both buffers
    ld de, #BUFFER_PTR_1
    ld b, LANE_POS_Y(iy)
    ld c, #LANE_PLAYER_SLOT_POS_X
    call getScreenPtr_32x16

    ex de, hl
    ld hl, #_spr_marker_player
    ld c, #_SPR_MARKER_PLAYER_W
    ld b, #_SPR_MARKER_PLAYER_H
    call drawSprite_32x16

    ld de, #BUFFER_PTR_2
    ld b, LANE_POS_Y(iy)
    ld c, #LANE_PLAYER_SLOT_POS_X
    call getScreenPtr_32x16

    ex de, hl
    ld hl, #_spr_marker_player
    ld c, #_SPR_MARKER_PLAYER_W
    ld b, #_SPR_MARKER_PLAYER_H
    call drawSprite_32x16
ret

;; ANTIGUO
;/////////////////////////////////////
;// Input:
;//     a - numero de lineas activas
;// Destroy: iy
;/////////////////////////////////////
;;sys_lanes_update::
;;    ld a, (#current_num_lanes)
;;    ld e, a
;;    ld iy, #lanes
;;    
;;    ;; TODO -> Comprobar que la linea no este en 0 y se esten moviendo los enemigos
;;    sys_lanes_update_loop:
;;        ;; Decrement the frame counter
;;        ld a, LANE_FRAME_COUNTER(iy)
;;        dec a
;;        ld LANE_FRAME_COUNTER(iy), a
;;        jr nz, sys_lanes_update_loop_next_lane
;;            ;; Reset the frame counter and decrement the line timer
;;            ld LANE_FRAME_COUNTER(iy), #LANE_FRAME_COUNTER_INIT_VALUE
;;            ld a, LANE_TIMER_VALUE(iy)
;;            dec a
;;            ld LANE_TIMER_VALUE(iy), a
;;            or a
;;            jr nz, test1
;;            ld a, LANE_TIMER_INIT_VALUE(iy)
;;            ld LANE_TIMER_VALUE(iy), a
;;test1:
;;            push de
;;            call z, sys_lanes_update_entities
;;            call sys_lanes_draw_lane_timer
;;            pop de
;;    sys_lanes_update_loop_next_lane:
;;        ld bc, #LANE_SIZE
;;        add iy, bc
;;        dec e
;;        jr nz, sys_lanes_update_loop
;;ret

sys_lanes_update_timers::
    ld a, (#level_generator_num_lanes)
    ld bc, #LANE_SIZE
    ld d, a
    ld iy, #lanes

    sys_lanes_update_timers_loop:
        ;; Only update the timer when the lane is not moving
        ld a, LANE_STATE(iy)
        and #LANE_STATE_MOVING
        jr nz, sys_lanes_update_timers_loop_next_lane
        
        ;; Decrement the frame counter and check if is 0
        ld a, LANE_FRAME_COUNTER(iy)
        dec a
        ld LANE_FRAME_COUNTER(iy), a
        jr nz, sys_lanes_update_timers_loop_next_lane
            ;; Reset the frame counter
            ld a, LANE_FRAME_COUNTER_INIT_VALUE(iy)
            ld LANE_FRAME_COUNTER(iy), a

            ;; Decrement the timer and check if is 0
            ld a, LANE_TIMER_VALUE(iy)
            dec a
            ld LANE_TIMER_VALUE(iy), a
            jr nz, sys_lanes_update_timers_loop_updating_timer
                ;; Set the state to moving and update the function callback
                ld LANE_STATE(iy), #LANE_STATE_MOVING
                ld LANE_UPDATE_FUNCTION_PTR_L(iy), #<sys_lanes_state_starting_to_move
                ld LANE_UPDATE_FUNCTION_PTR_H(iy), #>sys_lanes_state_starting_to_move

                jr sys_lanes_update_timers_loop_next_lane
            sys_lanes_update_timers_loop_updating_timer:
                ;; Set the state to updating timer and update the function callback
                ld LANE_STATE(iy), #LANE_STATE_UPDATING_TIMER
                ;; The callback to the update function can be set inside the lane moving callback
                ld LANE_UPDATE_FUNCTION_PTR_L(iy), #<sys_lanes_state_updating_timer
                ld LANE_UPDATE_FUNCTION_PTR_H(iy), #>sys_lanes_state_updating_timer
    sys_lanes_update_timers_loop_next_lane:
        add iy, bc
        dec d
    jr nz, sys_lanes_update_timers_loop
ret

sys_lanes_update::
    ld a, (#level_generator_num_lanes)
    ld d, a
    ld iy, #lanes
    
    sys_lanes_update_loop:
        ;; If the current lane is updating the counter go to the next lane
        ld a, LANE_STATE(iy)
        cp #LANE_STATE_UPDATING_COUNTER
        jr z, sys_lanes_update_loop_next_lane

        ;; Call the function pointer
        push de
        ld l, LANE_UPDATE_FUNCTION_PTR_L(iy)
        ld h, LANE_UPDATE_FUNCTION_PTR_H(iy)
        jp (hl) 
    sys_lanes_update_loop_callback_ret:
        pop  de
    sys_lanes_update_loop_next_lane:
        ld bc, #LANE_SIZE
        add iy, bc
        dec d
        jr nz, sys_lanes_update_loop
ret

;; ###### UPDATE CALLBACKS ######

sys_lanes_state_updating_timer:
    ;;Draw the lane and set the state to updating the frame counter so in the next frame this function is not called
    call sys_lanes_draw_lane_timer 
    ld LANE_STATE(iy), #LANE_STATE_UPDATING_COUNTER
jp sys_lanes_update_loop_callback_ret

sys_lanes_state_starting_to_move:    
    call sys_lanes_draw_lane_timer
    call sys_lanes_update_entities_ptrs

    ld LANE_FRAME_COUNTER(iy), #LANE_MOVE_ENTITY_FRAME_COUNTER   ;; Use the lane timer as an entity position counter
    ld LANE_TIMER_VALUE(iy), #LANE_MOVE_ENTITY_PER_FRAME_COUNTER ;; Use the lane timer as a frame counter 

    ;; Change the function pointer to move the entities
    ld LANE_UPDATE_FUNCTION_PTR_L(iy), #<sys_lanes_state_move_entities
    ld LANE_UPDATE_FUNCTION_PTR_H(iy), #>sys_lanes_state_move_entities

jp sys_lanes_update_loop_callback_ret

sys_lanes_state_move_entities:
    dec LANE_TIMER_VALUE(iy)
    jp nz, sys_lanes_update_loop_callback_ret
    ld LANE_TIMER_VALUE(iy), #LANE_MOVE_ENTITY_PER_FRAME_COUNTER

    ;; Decrement the frame counter
    ld a, LANE_FRAME_COUNTER(iy)
    dec a
    ld LANE_FRAME_COUNTER(iy), a
    
    cp #1
    jr nz, sys_lanes_state_move_entities_not_last_frame
        ;; If its the last frame change the function ptr and add the last frame state
        ld LANE_STATE(iy), #LANE_STATE_MOVING | #LANE_STATE_LAST_MOVING_FRAME
        ld LANE_UPDATE_FUNCTION_PTR_L(iy), #<sys_lanes_state_move_entities_last_frame
        ld LANE_UPDATE_FUNCTION_PTR_H(iy), #>sys_lanes_state_move_entities_last_frame
    sys_lanes_state_move_entities_not_last_frame:
        ld b, #(ANIM_ENEMY_WALKING)
        ld c, #0x00 ;; Set the mask to do nothing
        call sys_lanes_move_entities_once

jp sys_lanes_update_loop_callback_ret

sys_lanes_state_move_entities_last_frame:
    ld LANE_STATE(iy), #LANE_STATE_MOVING
    dec LANE_TIMER_VALUE(iy)
    jp nz, sys_lanes_update_loop_callback_ret
    
    ;; Set the mask to remove the moving state from the entity and add the just moved state
    ld b, #(ANIM_ENEMY_STANDING)
    ld c, #(ENTITY_STATE_MOVING | ENTITY_STATE_JUST_MOVED)  
    call sys_lanes_move_entities_once

    ;; Reset the frame counter and the timer value
    ld a, LANE_FRAME_COUNTER_INIT_VALUE(iy)
    ld LANE_FRAME_COUNTER(iy), a
    ld LANE_TIMER_VALUE(iy), #LANE_TIMER_INIT_VALUE
    ld LANE_STATE(iy), #LANE_STATE_UPDATING_COUNTER

    ;; If the last entity is not null kill it
    ld c, LANE_SLOTS_END_SLOT+LANE_SLOT_ENTITY_L(iy)
    ld b, LANE_SLOTS_END_SLOT+LANE_SLOT_ENTITY_H(iy)
    ld a, b
    or a, c
    jr z, sys_lanes_state_move_entities_last_frame_end
        ;; Load the entity in ix
        ld ix, #0
        add ix, bc

        ;; Mark the enemy as dead and remove the health
        ld b, #ANIM_ENEMY_DEATH_DAMAGE
        ld c, #ANIM_ENEMY_DEATH_DAMAGE_FRAMES
        call game_mark_enemy_as_dead
        xor a
        ld ENTITY_LIFE(ix), a

        ;; Set the entity slot to null
        ld LANE_SLOTS_END_SLOT+LANE_SLOT_ENTITY_L(iy), #0
        ld LANE_SLOTS_END_SLOT+LANE_SLOT_ENTITY_H(iy), #0

        call game_remove_player_health
    sys_lanes_state_move_entities_last_frame_end:
        call sys_lanes_draw_lane_timer
jp sys_lanes_update_loop_callback_ret

;;sys_lanes_update_one_lane::
;;    ld a, LANE_STATE(iy)
;;
;;    cp #LANE_STATE_UPDATING_TIMER
;;    jr nz, sys_lanes_update_one_lane_next_1
;;        call sys_lanes_draw_lane_timer 
;;        ld LANE_STATE(iy), #LANE_STATE_UPDATING_COUNTER
;;        ret
;;sys_lanes_update_one_lane_next_1:
;;    cp #LANE_STATE_MOVING
;;    jr nz, sys_lanes_update_one_lane_end
;;        call sys_lanes_move_entities
;;
;;sys_lanes_update_one_lane_end:
;;ret

;/////////////////////////////////////
;// Input:
;//     IY - Linea donde actualizar las entidades
;//      E - Mascara para hacer XOR con el estado de la entidad
;// Destroy: A, HL, BC, D, IX
;/////////////////////////////////////
sys_lanes_move_entities_once:
    push iy
    pop hl

    ;; Ignore the first entity as is always null
    ld de, #LANE_SLOT_SIZE
    add hl, de

    ld d, #LANE_NUM_SLOTS - 1 ;; -1 because the first one is ignored
    sys_lanes_move_entities_once_loop:
        push bc

        ;; Set bc to point to the entity 
        ld c, (hl) 
        inc hl
        ld b, (hl)
        inc hl 
        
        ;; If the entity is null go to the next entity
        ld a, c
        or a, b
        jr z, sys_lanes_move_entities_once_loop_null_entity

        ;; Set ix to point to the entity
        ld ix, #0
        add ix, bc

        ;; Increase the entity position by 1
        ld a, ENTITY_X(ix)
        inc a
        ld ENTITY_X(ix), a
        
        pop bc

        ;; Set the animation state
        ld ENTITY_ANIM_STATUS(ix), b

        ;; Apply the mask to the entity state
        ld a, ENTITY_STATE(ix)
        xor a, c
        ld ENTITY_STATE(ix), a

        jr sys_lanes_move_entities_once_loop_next_entity
    sys_lanes_move_entities_once_loop_null_entity:
        pop bc
    sys_lanes_move_entities_once_loop_next_entity:
        ;; Increse hl two times more because the cell size is 4 bytes
        inc hl
        inc hl
        dec d
        jr nz, sys_lanes_move_entities_once_loop
ret

;/////////////////////////////////////
;// Input:
;//     iy - Linea donde actualizar las entidades
;// Destroy: A, HL, BC, D, IX
;/////////////////////////////////////
sys_lanes_move_entities:
    ld a, LANE_FRAME_COUNTER(iy)
    dec a
    ld LANE_FRAME_COUNTER(iy), a
    jr z, sys_lanes_move_entities_reset
        ;; If the frame counter is not 0 Increase the entities x position by 1
        ld d, #LANE_NUM_SLOTS - 1
        push iy
        pop hl

        ;; Ignore the first entity as is always null
        ld bc, #LANE_SIZE
        add hl, bc
        sys_lanes_move_entities_loop:
            ;; Set bc to point to the entity 
            ld c, (hl) 
            inc hl
            ld b, (hl)
            inc hl 
            
            ;; If the entity is null go to the next entity
            ld a, c
            or a, b
            jr z, sys_lanes_move_entities_loop_next_entity

            ;; Set ix to point to the entity
            ld ix, #0
            add ix, bc

            ;; Increase the entity position by 1
            ld a, ENTITY_X(ix)
            inc a
            ld ENTITY_X(ix), a
        sys_lanes_move_entities_loop_next_entity:
            ;; Increse hl two times more because the cell size is 4 bytes
            inc hl
            inc hl
            dec d
            jr nz, sys_lanes_move_entities_loop
        ret

    sys_lanes_move_entities_reset:
        ;; Reset frame counter and timer
        ld LANE_FRAME_COUNTER(iy), #LANE_FRAME_COUNTER_INIT_VALUE
        ld a, LANE_FRAME_COUNTER_INIT_VALUE(iy)
        ld LANE_TIMER_VALUE(iy), a
        ld LANE_STATE(iy), #LANE_STATE_UPDATING_COUNTER

        ;; TODO: Check if the last entity is not null and destroy it
        push de
        call sys_lanes_draw_lane_timer
        pop de
ret


;/////////////////////////////////////
;// Input:
;//     iy - Linea donde actualizar las entidades
;// Destroy: 
;/////////////////////////////////////
sys_lanes_update_entities_ptrs::
    push iy
    pop ix
    push iy

    ld bc, #LANE_SLOTS_END_SLOT 
    add iy, bc                          ;; iy points to the last enemy on the lane (Always null space)
    ld bc, #LANE_SLOTS_END_SLOT - LANE_SLOT_SIZE
    add ix, bc                          ;; ix points to the previous entity pointed by iy
    
    ld bc, #-LANE_SLOT_SIZE
    ld d, #LANE_NUM_SLOTS - 1
    
    sys_lanes_update_entities_ptrs_loop:
        ;; Store the entity in de
        ld l, LANE_SLOT_ENTITY_L(ix)
        ld h, LANE_SLOT_ENTITY_H(ix)

        ;; Set the cell to null
        ld LANE_SLOT_ENTITY_L(ix), #00
        ld LANE_SLOT_ENTITY_H(ix), #00

        ;; Store the entity in the previous position
        ld LANE_SLOT_ENTITY_L(iy), l
        ld LANE_SLOT_ENTITY_H(iy), h

        ;;Check if the entity is null
        ld a, l
        or h
        jr z, sys_lanes_update_entities_ptrs_loop_next_entity

        ;;Store the entity in iy
        push iy
        push hl
        pop  iy

        ;;Set the entity state and animation state to moving and increment the lane x counter
        ld a, ENTITY_STATE(iy)
        or a, #ENTITY_STATE_MOVING
        ld ENTITY_STATE(iy), a
        ld ENTITY_ANIM_STATUS(iy), #ANIM_ENEMY_WALKING

        inc ENTITY_LANE_X(iy)
        
        ;;Restore the pointer
        pop iy

    sys_lanes_update_entities_ptrs_loop_next_entity: 
        add ix, bc
        add iy, bc

        dec d
        jr nz, sys_lanes_update_entities_ptrs_loop

    ;; Restore the lane
    pop iy
ret

timer_sprite_jump_table:
    .dw _spr_button_3, _spr_button_3, _spr_button_2, _spr_button_2, _spr_button_0, _spr_button_0

;/////////////////////////////////////
;// Input:
;//     iy - Linea donde dibujar
;// Destroy: A, HL, BC, DE, IX
;/////////////////////////////////////
sys_lanes_draw_lane_timer:
    ;; Test if the timer is odd
    ld c, LANE_TIMER_VALUE(iy)
    ld a, #01
    and a, c
    ret z

    sla c
    ld b, #0
    ld hl, #timer_sprite_jump_table
    add hl, bc

    ;; Load into hl the sprite
    ld a, (hl)
    inc hl
    ld h, (hl)
    ld l, a

    ;; Store 2 copies of the sprite
    push hl
    push hl

    ;; Calc button position
    ld de, #BUFFER_PTR_1
    ld c, #LANE_TIMER_POS_X
    ld b, LANE_POS_Y(iy)
    call getScreenPtr_32x16

    ;; Store the sprite in hl and the position in de
    pop de
    ex  de, hl

    ld c, #_SPR_BUTTON_W
    ld b, #_SPR_BUTTON_H
    call drawSprite_32x16

    ;; Calc char position
    ld de, #BUFFER_PTR_2
    ld c, #LANE_TIMER_POS_X
    ld b, LANE_POS_Y(iy)
    call getScreenPtr_32x16

    ;; Store the sprite in hl and the position in de
    pop de
    ex de, hl

    ld c, #_SPR_BUTTON_W
    ld b, #_SPR_BUTTON_H
    call drawSprite_32x16
ret