;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------

.include "./lanes.h.s"
.include "../macros.h.s"
.include "../constants.h.s"
.include "../manager/entity.h.s"
.include "../manager/entity_templates.h.s"
.include "../manager/entity_manager.h.s"
.include "../manager/levels.h.s"
.include "../cpctelera_includes.h.s"

.module level

.area _DATA

level_health_color_mask_table:
    .db 0b11000000, 0b11110000, 0b11111100, 0b11111111
level_health_color_table_ptrs:
    .dw level_health_color_table_color_red, level_health_color_table_color_cyan, level_health_color_table_color_yellow
level_health_color_table:
level_health_color_table_color_red:     .db 0b11000000, 0b00110000, 0b00001100, 0b00000011
level_health_color_table_color_cyan:    .db 0b10000000, 0b00100000, 0b00001000, 0b00000010
level_health_color_table_color_yellow:  .db 0b01000000, 0b00010000, 0b00000100, 0b00000001

level_enemy_type_table:
    .dw enemy_fire_tmpl
    .dw enemy_water_tmpl
    .dw enemy_electric_tmpl
    .dw enemy_mixed_tmpl
    .dw enemy_letter_tmpl
    .dw enemy_heart_tmpl
    

level_parse_command_table:
    .dw level_parse_command_end_command
    .dw level_parse_command_spawn_command
    .dw level_parse_command_wait_command

level_generator:

level_generator_current_lvl_ptr:
    .dw lvl1
level_generator_next_lvl_ptr:
    .blkw 1
level_generator_num_lanes::
    .ds 1
level_generator_min_hp:
    .ds 1
level_generator_max_hp_diff:
    .ds 1
level_generator_num_attacks::
    .ds 1
level_generator_num_attacks_minus_one:
    .ds 1
level_generator_current_num_level_left::
    .db 0
level_generator_current_num_level_right::
    .db 0
level_generator_num_lanes_finished::
    .db 0
level_generator_current_enemies_alive::
    .db 0
level_generator_lanes:
    .ds LEVEL_LANE_SIZE * 3 
    
.area _CODE

level_init::
    ld ix, #lvl1

    call level_load_level
ret

level_reset::
    ld hl, #lvl1
    ld (#level_generator_current_lvl_ptr), hl

    xor a
    ld (#level_generator_current_num_level_left),  a
    ld a, #1
    ld (#level_generator_current_num_level_right), a

ret

;/////////////////////////////////////
;// Input: 
;//     IX -> Level
;// Destroys: AF, BC, DE, HL
;/////////////////////////////////////
level_load_level::
    ld (#level_generator_current_lvl_ptr), ix

    xor a
    ld (#level_generator_num_lanes_finished), a
    ld (#level_generator_current_enemies_alive), a

    ;; Parse the first byte that contains the level data (Lines -> Attacks -> Min Health -> Max Health)
    ;; Num lines
    ld a, LEVEL_INIT_PARAMS(ix)
    ld b, a
    ld c, #LEVEL_DATA_MASK_2BITS
    and c
    ld (#level_generator_num_lanes), a

    ;; Num attacks/colors
    srl b
    srl b
    ld a, b
    and c
    ld (#level_generator_num_attacks), a
    dec a
    ld (#level_generator_num_attacks_minus_one), a

    ;; Min Health
    srl b
    srl b
    ld a, b
    and c
    inc a
    ld (#level_generator_min_hp), a

    ;; Max Health difference
    ld c, a 
    srl b
    srl b
    ld a, b
    inc a
    sub c
    ld (#level_generator_max_hp_diff), a

    ;; Load the address of the next lvl into hl
    ld l, LEVEL_NEXT_LEVEL_ADDRESS_L(ix)
    ld h, LEVEL_NEXT_LEVEL_ADDRESS_H(ix)
    ld (#level_generator_next_lvl_ptr), hl
    
    call level_load_lane_data
    call level_parse_lanes_first_command
ret

;/////////////////////////////////////
;// Destroys: A, HL
;// Sets the Z flag to 1 if there is no next level
;/////////////////////////////////////
level_has_next_level::
    ld hl, #level_generator_next_lvl_ptr
    ld a, (hl)
    inc hl
    or (hl)
ret

;/////////////////////////////////////
;// Input: 
;//     IX -> Level
;// Destroys: A, BC, DE, HL, IX, IY
;/////////////////////////////////////
level_load_lane_data:
    ld iy, #level_generator_lanes
    ld bc, #LEVEL_BUFFER_ADDRESES_START
    add ix, bc

    ;; Generate a random index to the lane (Stored in c and d)
    ld a, (#level_generator_num_lanes)
    ld b, a
    ld e, a
    dec b
    call random_clamped
    ld d, c

    level_load_lane_data_loop:
        ;; Set the pointer to the lane
        call sys_lanes_get_lane
        ld LEVEL_LANE_CURRENT_LANE_ADDRESS_L(iy), l
        ld LEVEL_LANE_CURRENT_LANE_ADDRESS_H(iy), h

        ;; Set the timer init value to the lane
        push iy

        push hl
        pop iy

        ld a, 2(ix)
        ld LANE_FRAME_COUNTER_INIT_VALUE(iy), a 

        pop iy

        ;; Set the pointer to the command buffer
        ld l, 0(ix)
        ld h, 1(ix)
        ld LEVEL_LANE_BUFFER_ADDRESS_L(iy), l
        ld LEVEL_LANE_BUFFER_ADDRESS_H(iy), h

        ld LEVEL_LANE_HAS_FINISHED(iy), #0

        ;; Set ix to point to the next level lane and iy to the next buffer pointer
        ld bc, #LEVEL_LANE_SIZE
        add iy, bc
        ld bc, #3
        add ix, bc

        ;; Increment the lane index and check if it has to be reseted (If the index is equal to the number of lanes)
        ld a, (#level_generator_num_lanes)
        inc d
        cp d
        jr nz, level_load_lane_data_dont_reset_index
            ld d, #0
        level_load_lane_data_dont_reset_index:
            ;; Load the index into c for the next iteration
            ld c, d
        dec e
        jr nz, level_load_lane_data_loop
ret

;; TODO -> Esto se puede hacer en la funcion de arriba probablemente
level_parse_lanes_first_command:
    ld iy, #level_generator_lanes
    ld a, (#level_generator_num_lanes)

    level_parse_lanes_first_command_loop:
        push af
        call level_parse_command
        pop af

        ld bc, #LEVEL_LANE_SIZE
        add iy, bc
        dec a
    jr nz, level_parse_lanes_first_command_loop
ret

level_load_next_level::
    ld a, (#level_generator_current_num_level_right)
    inc a
    cp #10
    jr nz, level_load_next_level_dont_reset
        ld a, (#level_generator_current_num_level_left)
        inc a
        ld (#level_generator_current_num_level_left), a
        xor a
    level_load_next_level_dont_reset:
        ld (#level_generator_current_num_level_right), a

    ld ix, (#level_generator_next_lvl_ptr)

    call level_load_level
ret

level_load_current_level::
    ld ix, (#level_generator_current_lvl_ptr)

    call level_load_level 
ret

level_level_has_been_completed::
    ld a, (#level_generator_current_enemies_alive)
    ld e, a
    ld a, (#level_generator_num_lanes_finished)
    sub a, e
    ld hl, #level_generator_num_lanes
    cp (hl)
ret

;/////////////////////////////////////
;// Input: 
;//     B -> N
;// Destroys: A, BC
;// Return: C -> Clamped number
;// Clamps a number between 0 and N when N >= 0 and N <= 3
;/////////////////////////////////////
random_clamped:
    ;; Return if N is 0
    ld c, #0
    ld a, #0
    cp b
    ret z

    push bc
    push de
    call cpct_getRandom_mxor_u8_asm
    pop de
    pop bc

    ld a, l
    and #0x03
    ld c, a
    sub b

    ret m
    ret z
    
    add a, #0x03
    and #0x03
    ld  c, a
ret

;/////////////////////////////////////
;// Input: 
;//     IX -> Entity
;// Destroys: A, BC, DE, HL
;// Entities must have the life field set to 0
;/////////////////////////////////////
level_set_entity_random_health::
    ;; Calculate the number of health elements (index to the color table)
    ld a, (#level_generator_max_hp_diff)
    ld b, a
    call random_clamped
    ld a, (#level_generator_min_hp)
    add a, c

    ;; Decrement E one time to use it as an index into the color table
    ld e, a
    dec e

    level_set_entity_random_health_loop:
        ;; Calculate the color of the health element (Index to the color_table_ptrs)
        ld a, (#level_generator_num_attacks_minus_one)
        ld b, a
        call random_clamped

        ;; C already has the index
        sla c               ;; Index *= 2
        ld b, #0
        ld hl, #level_health_color_table_ptrs
        add hl, bc

        ;;Load the pointed value of hl into hl
        ld a, (hl)
        inc hl
        ld h, (hl)
        ld l, a

        ;; E already has the index
        ld d, #0
        add hl, de

        ;; Load the color into the entity health bar
        ld a, (hl)
        or ENTITY_LIFE(ix)
        ld ENTITY_LIFE(ix), a

        dec e
        jp p, level_set_entity_random_health_loop
ret

;/////////////////////////////////////
;// Input: 
;//     IX -> Entity
;// Destroys: A, BC, HL
;// Entities must have the life field set to 0
;/////////////////////////////////////
level_set_entity_health:
    ;; Calculate the number of health elements (index to the color table)
    ld a, (#level_generator_max_hp_diff)
    ld b, a
    call random_clamped
    ld a, (#level_generator_min_hp)
    add a, c

    ;; Decrement A one time to use it as an index into the color mask table
    dec a

    ld b, #0
    ld c, a
    ld hl, #level_health_color_mask_table
    add hl, bc

    ld b, (hl)
    ld a, ENTITY_LIFE(ix)
    and b
    ld ENTITY_LIFE(ix), a
ret


;/////////////////////////////////////
;// Input: 
;// Destroys: BC, IX, IY
;/////////////////////////////////////
level_update::
    ld iy, #level_generator_lanes
    ld a, (#level_generator_num_lanes)
    ld d, a
    
    level_update_loop:
        ;; If the lane has finished (1), go to the next one
        ld a, LEVEL_LANE_HAS_FINISHED(iy)
        or a
        jr nz, level_update_loop_next_lane

        ld l, LEVEL_LANE_CURRENT_CMD_PTR_L(iy)
        ld h, LEVEL_LANE_CURRENT_CMD_PTR_H(iy)

        push de
        jp (hl)
    level_update_loop_end_command:
        pop  de

    level_update_loop_next_lane:
        ld  bc, #LEVEL_LANE_SIZE
        add iy, bc
        dec d
        jr nz, level_update_loop
ret

;/////////////////////////////////////
;// Input: 
;//     IY -> Level Lane to parse
;// Destroys: IX, IY
;/////////////////////////////////////
level_parse_command:
    ;; Get the command byte, store a copy in d and update the command bufffer
    ld l, LEVEL_LANE_BUFFER_ADDRESS_L(iy)
    ld h, LEVEL_LANE_BUFFER_ADDRESS_H(iy)

    ld a, (hl)
    ld d, a  

    inc hl
    ld LEVEL_LANE_BUFFER_ADDRESS_L(iy), l
    ld LEVEL_LANE_BUFFER_ADDRESS_H(iy), h
    
    ;; Parse the command type and calculate the index
    and a, #LEVEL_DATA_MASK_2BITS
    add a, a                    ;; Index *= 2
    ld  b, #0
    ld  c,  a

    ld hl, #level_parse_command_table
    add hl, bc

    ;; Load the function into hl and jump
    ld a, (hl)
    inc hl
    ld h, (hl)
    ld l, a

    ;; Remove the command type bits from the command data
    srl d
    srl d

    jp (hl)

;; No need to ret. The parse command functions have the ret instr 

;/////////////////////////////////////
;// Input: 
;//     IX -> Level generator
;//     IY -> Level lane
;//      D -> Command Data
;/////////////////////////////////////
level_parse_command_end_command:
    ld LEVEL_LANE_HAS_FINISHED(iy), #1
    ld a, (#level_generator_num_lanes_finished)
    inc a
    ld (#level_generator_num_lanes_finished), a
ret

;/////////////////////////////////////
;// Input: 
;//     IX -> Level generator
;//     IY -> Lane to parse
;/////////////////////////////////////
level_parse_command_wait_command:
    ;; Load the num waits into the data1 byte and the command function
    inc d
    ld LEVEL_LANE_CURRENT_CMD_DATA1(iy), d
    ld LEVEL_LANE_CURRENT_CMD_PTR_L(iy), #<level_command_wait
    ld LEVEL_LANE_CURRENT_CMD_PTR_H(iy), #>level_command_wait
ret

;/////////////////////////////////////
;// Input: 
;//     IX -> Level generator
;/////////////////////////////////////
level_parse_command_spawn_command:
    ;; Get the num enemies data
    ld a, d
    and a, #LEVEL_DATA_MASK_2BITS

    inc a
    ;; Load the num enemies into the data1 byte (1-4)
    ld LEVEL_LANE_CURRENT_CMD_DATA1(iy), a

    ;; Get the enemy type data
    srl d
    srl d

    ;; Calculate the index into the enemy type table
    ld a, d
    add a, a
    ld b, #0
    ld c,  a

    ld hl, #level_enemy_type_table
    add hl, bc

    ;; Load the template into the data2 and data3 bytes
    ld a, (hl)
    ld LEVEL_LANE_CURRENT_CMD_DATA2(iy), a
    inc hl
    ld a, (hl)
    ld LEVEL_LANE_CURRENT_CMD_DATA3(iy), a

    ;; Load the command function ptr
    ld LEVEL_LANE_CURRENT_CMD_PTR_L(iy), #<level_command_spawn
    ld LEVEL_LANE_CURRENT_CMD_PTR_H(iy), #>level_command_spawn
ret

;/////////////////////////////////////
;// Input: 
;//     IY -> Level Lane
;/////////////////////////////////////
level_command_wait:
    call load_lane_from_level_lane

    ;; Check if it's the last frame moving
    ld a, LANE_STATE(ix)
    and  #LANE_STATE_LAST_MOVING_FRAME
    jp z, level_update_loop_end_command

    ld a, LEVEL_LANE_CURRENT_CMD_DATA1(iy)
    dec a
    ld LEVEL_LANE_CURRENT_CMD_DATA1(iy), a
    jp nz, level_update_loop_end_command

    call level_parse_command

;; Jump back to the update function
jp level_update_loop_end_command

;/////////////////////////////////////
;// Input: 
;//     IY -> Level Lane
;/////////////////////////////////////
level_command_spawn:
    call load_lane_from_level_lane

    ;; Dont spawn and enemy if the first slot entity is not null  
    ld a, LANE_SLOT_ENTITY_L(ix)
    ld b, LANE_SLOT_ENTITY_H(ix)
    or a, b
    jp nz, level_update_loop_end_command
    
    ;; Dont spawn and enemy is the lane is moving
    ld a, LANE_STATE(ix)
    cp #LANE_STATE_MOVING
    jp z, level_update_loop_end_command

    ;; Create the entity
    ld a, (#level_generator_current_enemies_alive)
    inc a
    ld (#level_generator_current_enemies_alive), a

    push iy
    push ix

    call entity_man_create_entity
    ;; Set hl to point to the template ptr
    ld l, LEVEL_LANE_CURRENT_CMD_DATA2(iy)
    ld h, LEVEL_LANE_CURRENT_CMD_DATA3(iy)
    ldir
    
    ;; iy points to the lane and ix points to the new entity
    pop iy    

    ;; Set the position of the entity into the first slot
    ld a, LANE_POS_Y(iy)
    sub #4
    ld ENTITY_Y(ix), a
    ld ENTITY_X(ix), #LANE_SLOT_START_POS_X

    ld a, LANE_INDEX_Y(iy)
    ld ENTITY_LANE_Y(ix), a
    ld ENTITY_LANE_X(ix), #0

    ;; Set a random healthbar if the entity has the state otherwise a masked health
    ld a, ENTITY_STATE(ix)
    and #ENTITY_STATE_RANDOM_HEALTHBAR
    call nz, level_set_entity_random_health
    call z, level_set_entity_health

    ;; Store the entity inside the lane
    push ix
    pop bc

    ld LANE_SLOTS_START_SLOT(iy),   c
    ld LANE_SLOTS_START_SLOT+1(iy), b

    ;; iy points to the level lane
    pop iy

    ;; Substract 1 from the command data -> If the count is 0 the command has finished -> Parse the next command
    ld a, LEVEL_LANE_CURRENT_CMD_DATA1(iy)
    dec a
    ld LEVEL_LANE_CURRENT_CMD_DATA1(iy), a
    jp nz, level_update_loop_end_command

    call level_parse_command

;; Jump back to the update function
jp level_update_loop_end_command

;/////////////////////////////////////
;// Input: 
;//     IY -> Level Lane
;/////////////////////////////////////
load_lane_from_level_lane:
    ;; Load the lane into ix
    ld c, LEVEL_LANE_CURRENT_LANE_ADDRESS_L(iy)
    ld b, LEVEL_LANE_CURRENT_LANE_ADDRESS_H(iy)
    ld ix, #0
    add ix, bc
ret