;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------

.module render

.include "cpctelera.h.s"
.include "../cpctelera_includes.h.s"
.include "../manager/entity.h.s"
.include "../manager/entity_manager.h.s"
.include "../macros.h.s"
.include "render_cpctelera32x16.h.s"

.area _CODE

.globl _main_palette

; DCBA 3210   0  2  1  3
;              A  C  B  D

; 1B3D
; 0A2C Color de la vida

; Verde     idx 10   -> 0b1010 -> 0101 0000
; Verde     idx 03   -> 0b0011 -> 1100 1100


; 00 NADA
BLACK_IDX       = 0b00000000
BLACK_COLOR     = 0b00000000
; 01 Amarillo  idx 13   -> 0b1101 -> 0101 0001
YELLOW_IDX      = 0b00000001
YELLOW_COLOR    = 0b01010001
; 10 Cian      idx 12    -> 0b1100 -> 0001 0001
CYAN_IDX        = 0b00000010
CYAN_COLOR      = 0b00010001
; 11 Rojo      idx 4    -> 0b0100 -> 0001 0000
RED_IDX         = 0b00000011
RED_COLOR       = 0b00010000

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; VARIABLES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_active_buffer:
    .dw 0xC000
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; PRIVATE FUNCTIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;/////////////////////////////////////
    ;// FUNCTION
    ;//   DIRECCION 0XBC -> CRTC Registro
    ;// Input:
    ;//     D - Register 
    ;//     E - Val
    ;/////////////////////////////////////
    set_crtc_screen:
        ld bc, #0xBC00      ;; BC selecciono un registro
        out (c), d
        inc b               ;; B = bd -> bc+1
        out (c), e
    ret
    ;/////////////////////////////////////
    ;/////////////////////////////////////
    ;// FUNCTION
    ;//   DIRECCION 0XBC -> CRTC Registro
    ;/////////////////////////////////////
    change_screen_size:

        ;; R1 = 32 (Ancho en caracteres = 64 bytes)
        ld de, #0x0120
        call set_crtc_screen

        ;; R2 = 42 (Inicio de HSYNC)
        ld de, #0x022A
        call set_crtc_screen

        ;; R6 = 16 (Alto en caracteres = 32 bytes)
        ld de, #0x0610
        call set_crtc_screen
        ;; R7 = 26 (Inicio de VSYNC)
        ld de, #0x071A
        call set_crtc_screen

        ret
    ;/////////////////////////////////////

    


    ;/////////////////////////////////////
    ;// FUNCTION
    ;//     Change between C000 & C400
    ;// Input:
    ;/////////////////////////////////////
    sys_render_swap_buffers::
        f_change_screen = . + 1
        jp change_screen_to_C400    
    change_screen_to_C400:
        ;; Cambiamos a dibujar en C400 
            ld de, #0x0C32
            call set_crtc_screen
        ;;
        ld a, #0xC0             ;;/ cambiamos screen_ptr de getScreenPtr a C400
        ld (back_buffer), a     ;;\

        ld hl, #change_screen_to_C000
        ld (f_change_screen), hl

        ret
    change_screen_to_C000:
        ;; Cambiamos a dibujar en C000 
            ld de, #0x0C30
            call set_crtc_screen
        ;; 
        ld a, #0xC4
        ld (back_buffer), a

        ld hl, #change_screen_to_C400
        ld (f_change_screen), hl

        ret
    ;/////////////////////////////////////
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

sys_render_one_entity::
    ld e, ENTITY_PREV_PTR2_L(iy)
    ld d, ENTITY_PREV_PTR2_H(iy)
    ld a, #0
    ld c, ENTITY_W(iy)
    ld b, ENTITY_H(iy) 
    call drawSolidBox_32x16

; Get the screen ptr
back_buffer = . + 2 ;;ld ( 1 byte ) cambiar C4 por C0 y viceversa
    ld de, #0xC400      ;; 1o dibujamos donde no se ve
    ld c, ENTITY_X(iy)
    ld b, ENTITY_Y(iy)
    call getScreenPtr_32x16

    ld a, ENTITY_PREV_PTR1_L(iy)
    ld ENTITY_PREV_PTR2_L(iy), a
    ld a, ENTITY_PREV_PTR1_H(iy)
    ld ENTITY_PREV_PTR2_H(iy), a

    ;;Store the new previous pointer
    ld ENTITY_PREV_PTR1_L(iy), l
    ld ENTITY_PREV_PTR1_H(iy), h

    ; Check if the entity is dead
    ld a, ENTITY_STATE(iy)
    and #ENTITY_STATE_DEAD
    jr z, entity_is_alive
    ;; Entity is dead -> Draw a solid black box
    
    ;; Remove the current position
    ld c, ENTITY_W(iy)
    ld b, ENTITY_H(iy)
    ld a, #0
    ex de, hl
    call drawSolidBox_32x16

    ;; Remove the previous position
    ld c, ENTITY_W(iy)
    ld b, ENTITY_H(iy)
    ld a, #0
    ld e, ENTITY_PREV_PTR2_L(iy)
    ld d, ENTITY_PREV_PTR2_H(iy)
    call drawSolidBox_32x16

    jp sys_render_one_entity_end 

entity_is_alive:
    ; Entity is alive -> Draw the entity
    push hl
    
    ld e, ENTITY_SPRITE_L(iy)
    ld d, ENTITY_SPRITE_H(iy)
    ld c, ENTITY_W(iy)
    ld b, ENTITY_H(iy)
    ex de, hl
    call drawSprite_32x16
    
    pop hl ; Recuperamos puntero
    ;jr sys_render_one_entity_end 

    ;;Check if the entity has life
    ld a, ENTITY_LIFE(iy)
    or a
    jr z, sys_render_one_entity_end

entity_draw_life:
    ;; Get pointer to draw
    ld d, #5
    ld e, #0
    push de
    

    push hl
    pop de
    ld bc, #0x800    ; -line size
    add hl, bc      ;/ y-=1 

    ld b, a             ; b = enemy life
    ld c, #0b00000011   ; c = bit mask

    while_entity_draw_life:
    pop af
    dec a
    push af
    or a
    jr z, end_while_entity_draw_life
    
    ld a,b
    and c 
    
    cp #BLACK_IDX
    jr nz, entity_draw_life_not_black
    ld a, #BLACK_COLOR
    jr entity_draw_life_draw
    entity_draw_life_not_black:
    
    cp #YELLOW_IDX
    jr nz, entity_draw_life_not_yellow
    ld a, #YELLOW_COLOR
    jr entity_draw_life_draw
    entity_draw_life_not_yellow:
    
    cp #CYAN_IDX
    jr nz, entity_draw_life_not_cyan
    ld a, #CYAN_COLOR
    jr entity_draw_life_draw
    entity_draw_life_not_cyan:
    

    ld a, #RED_COLOR

    entity_draw_life_draw:
    ld (hl), a
    ld (de), a
    inc hl
    inc de
    srl b
    srl b
    jr while_entity_draw_life

    end_while_entity_draw_life:
    pop af

sys_render_one_entity_end:
ret

sys_render_update::
    SET_CALL_INSTRUCTION_ADDRESS #entity_man_for_all_matching_function_ptr, #sys_render_one_entity
    ld a, #COMPONENT_RENDER
    call entity_man_for_all_matching
ret

sys_render_reset_double_buffer::
    ;; Change crtc pointer to point to C000
    ld de, #0x0C30
    call set_crtc_screen

    ;; Change the sys_render_one_entity buffer to point to C400
    ld a, #0xC4             
    ld (back_buffer), a
ret

sys_render_init::
    ;Se hace en image conversion
    ld c, #0
    call cpct_setVideoMode_asm

    call change_screen_size

    ld hl, #_main_palette
    ld de, #16
    call cpct_setPalette_asm

    cpctm_setBorder_asm HW_BLACK
ret