;;------------------------------------------------------------------------------
;; This file is part of Farming Spirits.
;;
;; Farming Spirits is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published 
;; by the Free Software Foundation, either version 3 of the License, 
;; or (at your option) any later version.
;;
;; Farming Spirits is distributed in the hope that it will be useful, but WITHOUT 
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
;; FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
;; for more details.
;;
;; You should have received a copy of the GNU General Public License 
;; along with Farming Spirits. If not, see https://www.gnu.org/licenses/. 
;;------------------------------------------------------------------------------

.include "cpctelera.h.s"
.include "../manager/game.h.s"
.include "../manager/entity.h.s"
.include "../manager/entity_manager.h.s"
.include "../manager/animations.h.s"
.include "../manager/entity_templates.h.s"
.include "../constants.h.s"
.include "../system/sounds.h.s"
.include "../cpctelera_includes.h.s"
.include "./level.h.s"

IS_PRESSED_TRUE = 0x01
IS_PRESSED_FALSE = 0x00
Right_Is_Pressed:
    .db 0x00

Left_Is_Pressed:
    .db 0x00


sys_update_behaviour_call_hl: jp (hl)

sys_update_behaviour_update_one_entity:
    ld l, ENTITY_UPDATE_BEHAVIOUR_L(iy)
    ld h, ENTITY_UPDATE_BEHAVIOUR_H(iy)
    call sys_update_behaviour_call_hl
ret

sys_update_behaviour_update::
    SET_CALL_INSTRUCTION_ADDRESS #entity_man_for_all_matching_function_ptr, #sys_update_behaviour_update_one_entity
    ld a, #COMPONENT_UPDATE
    call entity_man_for_all_matching
ret

sys_update_behaviour_entity_dying::
    ld a, ENTITY_UPDATE_COUNTER(iy)
    dec a
    jr z, sys_update_behaviour_entity_dying_death
        ld ENTITY_UPDATE_COUNTER(iy), a       
        ret
    sys_update_behaviour_entity_dying_death:
        ;; Mark the entity as dead
        ld ENTITY_STATE(iy), #ENTITY_STATE_DEAD
ret

sys_update_behaviour_enemy_dying::
    ld a, ENTITY_UPDATE_COUNTER(iy)
    dec a
    jr z, sys_update_behaviour_enemy_dying_death
        ld ENTITY_UPDATE_COUNTER(iy), a       
        ret
    sys_update_behaviour_enemy_dying_death:
        ;; Mark the entity as dead and decrement the enemies alive counter
        ld a, (#level_generator_current_enemies_alive)
        dec a
        ld (#level_generator_current_enemies_alive), a
        ld ENTITY_STATE(iy), #ENTITY_STATE_DEAD
ret


sys_update_behaviour_player_bullet::
    ld a, ENTITY_X(iy)
    ld b, ENTITY_BULLET_VX(iy)
    add a, b
    ld ENTITY_X(iy), a

    ;; If bullet->x > MIN_POSITION -> ret
    cp #BULLET_MIN_X_POSITION
    ret p

    ;; Mark bullet as dead
    ld a, ENTITY_STATE(iy)
    or a, #ENTITY_STATE_DEAD
    ld ENTITY_STATE(iy), a
ret

sys_update_behaviour_enemy_bullet::
    ld a, ENTITY_X(iy)
    ld b, ENTITY_BULLET_VX(iy)
    add a, b
    ld ENTITY_X(iy), a

    ;; If bullet->x < MAX_POSITION -> ret
    cp #BULLET_MAX_X_POSITION
    ret m

    ;; Mark bullet as dead
    ld a, ENTITY_STATE(iy)
    or a, #ENTITY_STATE_DEAD
    ld ENTITY_STATE(iy), a
ret

sys_update_behaviour_enemy_letter::
    ld a, ENTITY_STATE(iy)
    and #ENTITY_STATE_JUST_MOVED
    ret z

    ;; Remove the just moved state
    ld a, ENTITY_STATE(iy)
    and #~ENTITY_STATE_JUST_MOVED
    ld ENTITY_STATE(iy), a

    ;; Dont shot if it is close to the player
    ld a, ENTITY_LANE_X(iy)
    cp #LANE_NUM_SLOTS - 2
    ret z

    ;; Create a card bullet
    ld hl, #enemy_letter_bullet_tmpl
    call game_create_enemy_bullet
ret



sys_update_behaviour_player::
    ;; Cambiar tipo bala
    ; (Se puede hacer independientemente de su estado)
    call sys_update_input_change_bullet

    ;; Check state
    ld a, ENTITY_STATE(iy)
    
    ;; Mover enemigo
    cp #ENTITY_STATE_MOVING
    jp z, sys_update_movement_player ; TRUE

    ;; Disparar
    cp #ENTITY_STATE_SHOOTING
    jp z, sys_update_shoot_player ;TRUE

    ;; Comprobar input
    or a ; = cp #ENTITY_STATE_DEFAULT
    jp z, sys_update_input_player ;TRUE
;;

sys_update_input_change_bullet:
    ; 0 no presionado
    ; 1 presionado

    ld hl, #Key_O
    call cpct_isKeyPressed_asm
    ld hl, #Right_Is_Pressed
    jr nz, Right_Pressed

    ld hl, #Joy0_Left
    call cpct_isKeyPressed_asm
    ld hl, #Right_Is_Pressed
    jr z, Right_NotPressed

    Right_Pressed:
        ld a, (hl)
        or a
        jr nz, Right_End ; Presionado
        push hl
        call game_set_next_attack_type
        pop hl
        ld (hl), #IS_PRESSED_TRUE
        jr Right_End
    Right_NotPressed:
    ld (hl), #IS_PRESSED_FALSE
    Right_End:
    ;ld ENTITY_PLAYER_CHANGE_BULLET(iy), a

    ld hl, #Key_P
    call cpct_isKeyPressed_asm
    ld hl, #Left_Is_Pressed
    jr nz, Left_Pressed
    Left_Ckeck_Joy:
    ld hl, #Joy0_Right
    call cpct_isKeyPressed_asm
    ld hl, #Left_Is_Pressed
    jr z, Left_NotPressed
    Left_Pressed:
        ld a, (hl)
        or a
        jr nz, Left_End ; Presionado
        push hl
        call game_set_previous_attack_type
        pop hl
        ld (hl), #IS_PRESSED_TRUE
        jr Left_End
    Left_NotPressed:
    ld (hl), #IS_PRESSED_FALSE
    Left_End:
ret


sys_update_input_player::
    ld hl, #Key_Space
    call cpct_isKeyPressed_asm
    jr nz, Space_Pressed

    ld hl, #Joy0_Fire1
    call cpct_isKeyPressed_asm
    jr z, Space_NotPressed
    ;call sys_sounds_play_dead
    ;call sys_sounds_play_hit
    Space_Pressed:
        call sys_sounds_play_spell
        ld ENTITY_UPDATE_COUNTER(iy), #ANIM_PLAYER_SHOOTING_FRAMES
        ld ENTITY_STATE(iy), #ENTITY_STATE_SHOOTING
        ld ENTITY_ANIM_STATUS(iy), #ANIM_PLAYER_SHOOTING
        call game_create_player_bullet
        ret
    Space_NotPressed:
    ld ENTITY_ANIM_STATUS(iy), #ANIM_PLAYER_STANDING    ;; TODO:  

    ;; Left - Right Movement
    ld e, #0


    ld hl, #Key_Q
    call cpct_isKeyPressed_asm
    jr nz, Up_Pressed

    ld hl, #Joy0_Up
    call cpct_isKeyPressed_asm
    jr z, Up_NotPressed
    Up_Pressed:
        ld  a, ENTITY_LANE_Y(iy)
        or a
        jr z, Up_NotPressed
        dec a
        ld  ENTITY_LANE_Y(iy), a

        dec e ; / e -= 1 pix
        dec e ; | e -= 2 pix
        ;dec e ; | e -= 3 pix
        dec e ; \ e -= 4 pix
        ld ENTITY_ANIM_STATUS(iy), #ANIM_PLAYER_UP
    Up_NotPressed:
    

    ld hl, #Key_A
    call cpct_isKeyPressed_asm
    jr nz, Down_Pressed

    ld hl, #Joy0_Down
    call cpct_isKeyPressed_asm
    jr z, Down_NotPressed
    Down_Pressed:
        ld  c, ENTITY_LANE_Y(iy)
        inc c
        ld a, (#level_generator_num_lanes)
        cp  c
        jr z, Down_NotPressed
        ld  ENTITY_LANE_Y(iy), c

        inc e ; / e += 1 pix
        inc e ; | e += 2 pix
        ;inc e ; | e += 3 pix
        inc e ; \ e += 4 pix
        ld ENTITY_ANIM_STATUS(iy), #ANIM_PLAYER_DOWN
    Down_NotPressed:

    ld a,e
    or a
    ret z
    
    ld ENTITY_VELOCITY(iy), e
    ld ENTITY_UPDATE_COUNTER(iy), #LANE_SLOT_SEPARATION_Y
    ld ENTITY_STATE(iy), #ENTITY_STATE_MOVING
    
ret

sys_update_shoot_player:
    ld a, ENTITY_UPDATE_COUNTER(iy)
    dec a
    ld ENTITY_UPDATE_COUNTER(iy), a
    or a
    ret nz ; a->CONTADOR_Y !=0

    sys_update_shoot_player_reset: ; a->CONTADOR_Y ==0
    ld ENTITY_STATE(iy), #ENTITY_STATE_DEFAULT
    ld ENTITY_UPDATE_COUNTER(iy), #ANIM_PLAYER_STANDING
ret

sys_update_movement_player:
    ld a, ENTITY_UPDATE_COUNTER(iy)
    dec a
    ld ENTITY_UPDATE_COUNTER(iy), a
    or a
    jr z, sys_update_movement_player_reset ; a->CONTADOR_Y ==0
    
    ld a, ENTITY_Y(iy)
    add ENTITY_VELOCITY(iy)
    ld ENTITY_Y(iy), a
    ret
    
    sys_update_movement_player_reset:
    ld ENTITY_VELOCITY(iy), a ; ENT_VY = a->0
    ld ENTITY_STATE(iy), #ENTITY_STATE_DEFAULT
    ld ENTITY_ANIM_STATUS(iy), #ANIM_PLAYER_STANDING

ret
